/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "resourcefile.h"
#include "sdlthemeproperties.h"

/****************************************
 *
 * TSize encode/decode
 *
 ***************************************/

template<>
bool Decode(const std::string& aEncodedData, sdl::TSize& aRawResourceItem)
{
	bool success = true;
	std::vector<std::string> stringList;
	Decode(aEncodedData, stringList);
	if (stringList.size() == 2)
	{
		aRawResourceItem = sdl::TSize(atoi(stringList.at(0).c_str()), atoi(stringList.at(1).c_str()));
	}
	else
	{
		success = false;
	}
	return success;
}


template<>
void Encode(std::string& aEncodedData, sdl::TSize& aRawResourceItem)
{
	std::vector<std::string> stringList = 
	{
		std::to_string(aRawResourceItem.iWidth),
		std::to_string(aRawResourceItem.iHeight)		
	};
	::Encode(aEncodedData, stringList);
}

/****************************************
 *
 * TPoint encode/decode
 *
 ***************************************/

template<>
bool Decode(const std::string& aEncodedData, sdl::TPoint& aRawResourceItem)
{
	bool success = true;
	std::vector<std::string> stringList;
	Decode(aEncodedData, stringList);
	if (stringList.size() == 2)
	{
		aRawResourceItem = sdl::TPoint(atoi(stringList.at(0).c_str()), atoi(stringList.at(1).c_str()));
	}
	else
	{
		success = false;
	}
	return success;
}


template<>
void Encode(std::string& aEncodedData, sdl::TPoint& aRawResourceItem)
{
	std::vector<std::string> stringList = 
	{
		std::to_string(aRawResourceItem.iX),
		std::to_string(aRawResourceItem.iY)		
	};
	::Encode(aEncodedData, stringList);
}

/****************************************
 *
 * TRect encode/decode
 *
 ***************************************/

template<>
bool Decode(const std::string& aEncodedData, sdl::TRect& aRawResourceItem)
{
	bool success = true;
	std::vector<std::string> stringList;
	Decode(aEncodedData, stringList);
	if (stringList.size() == 4)
	{
		aRawResourceItem.SetPosition(sdl::TPoint(atoi(stringList.at(0).c_str()), atoi(stringList.at(1).c_str())));
		aRawResourceItem.SetSize(sdl::TSize(atoi(stringList.at(2).c_str()), atoi(stringList.at(3).c_str())));
	}
	else
	{
		success = false;
	}
	return success;
}


template<>
void Encode(std::string& aEncodedData, sdl::TRect& aRawResourceItem)
{
	std::vector<std::string> stringList = 
	{
		std::to_string(aRawResourceItem.X()), 
		std::to_string(aRawResourceItem.Y()),
		std::to_string(aRawResourceItem.Width()),
		std::to_string(aRawResourceItem.Height())		
	};
	::Encode(aEncodedData, stringList);
}


/****************************************
 *
 * THex encode/decode
 *
 ***************************************/

template<>
bool Decode(const std::string& aEncodedData, THex& aRawResourceItem)
{
	return ::Decode<int>(aEncodedData, aRawResourceItem.iValue);
}

template<>
void Encode(std::string& aEncodedData, THex& aRawResourceItem)
{
	std::stringstream stream;
	stream << "0x" << std::hex << (int) aRawResourceItem;
	aEncodedData.assign( stream.str() );
}


/****************************************
 *
 * TColor encode/decode
 *
 ***************************************/

template<>
bool Decode(const std::string& aEncodedData, sdl::TColor& aRawResourceItem)
{
	return ::Decode<int>(aEncodedData, aRawResourceItem.iColor);
}


template<>
void Encode(std::string& aEncodedData, sdl::TColor& aRawResourceItem)
{
	THex hex(aRawResourceItem.iColor);
	::Encode<THex>(aEncodedData, hex);
}
