/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __FAVS_GAMES_H
#define __FAVS_GAMES_H

#include "sdlimenulistener.h"
#include "types.h"

class Menu;

namespace FavsGames
{
	class Model;
	
	class SettingsMenuController : public sdl::IMenuItemListener
	{
	public:
		SettingsMenuController(Model& aModel) : iModel(aModel) { }
		
	private: // from IMenuItemListener
		virtual void MenuItemHighlighted(const sdl::MenuItem& aMenuItem) override { }
		virtual void MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand) override;
		virtual void MenuListChanged(sdl::Menu& aMenu) override {}
		
		void CmdClearFavs();
		void CmdConfigureFavsMenu();
		
		Model& iModel;
	};

	class FavsGamesMenuController : public sdl::IMenuItemListener
	{
	public:
		FavsGamesMenuController (Model& aModel) : iModel(aModel) { }
		
	private: // from IMenuItemListener
		virtual void MenuItemHighlighted(const sdl::MenuItem& aMenuItem) override { }
		virtual void MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand) override;
		virtual void MenuListChanged(sdl::Menu& aMenu) override {}
		
		Model& iModel;
	};
	
	
	class Model
	{
	public:
		static const std::string KMenuName;

		Model() : iFavsGamesMenuController(*this), iSettingsMenuController(*this) { }
		void Construct();
		sdl::Menu* GetMenu() { return iMenu; }

		FavsGamesMenuController&	GetFavsGamesMenuController(){ return iFavsGamesMenuController ; }
		SettingsMenuController&		GetSettingsMenuController()	{ return iSettingsMenuController; }
	private:
		friend FavsGamesMenuController;
		friend SettingsMenuController;
		
		void Reset();
		void Reconfigure();

		void Add(const sdl::MenuItem& aMenuItem);
		void Delete(const sdl::MenuItem& aMenuItem);
		void Persist();

		sdl::Menu*	iMenu = nullptr; // don't own it

		static std::string KFileName;
		
		FavsGamesMenuController iFavsGamesMenuController ;
		SettingsMenuController	iSettingsMenuController;
	};

}
#endif // __FAVS_GAMES_H