/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "gamescontainer.h"

#include "platform.h"
#include "fileutils.h"

/*************************************************
 * 
 * GamesIter_const
 * 
 *************************************************/ 

bool GameFilterConfig::IsGameSupported(const TGame& aGame) const
{
	bool isGameSupported = true;
	
	if (iMaxButtons != -1)
	{
		if (aGame.iButtons >  iMaxButtons)
			isGameSupported = false;
	}
	
	if (iSimultaneousPlayers)
	{
		isGameSupported = true;
		if (aGame.iSimulatenousPlay && (iMaxPlayers != -1) && (aGame.iPlayers > iMaxPlayers))
			isGameSupported = false;
	}
	
	return isGameSupported;
}

/*************************************************
 * 
 * GamesIter_const
 * 
 *************************************************/ 

GamesIter_const::GamesIter_const(const GamesContainer& aGamesContainer, bool aBegin) : iGamesContainer(aGamesContainer) 
{ 
	if (aBegin)
		Begin();
	else
		End();
}

GamesIter_const::GamesIter_const(const GamesContainer& aGamesContainer, const std::string& aGameName) : iGamesContainer(aGamesContainer) 
{
	iGameId.clear();
	auto it = iGamesContainer.iGameMap.find(aGameName);
	if (it != iGamesContainer.iGameMap.end())
		if (iGamesContainer.IsGameSupported(it->second))
			iGameId = it->first;
}

void GamesIter_const::FindNextSupportedGame(const std::string aGameName)
{
	auto it = iGamesContainer.iGameMap.find(aGameName);
	while (++it != iGamesContainer.iGameMap.end())
	{
		if (iGamesContainer.IsGameSupported(it->second))
		{
			iGameId = it->first;
			break;
		}
	}
}

void GamesIter_const::Begin() 
{
	iGameId.clear();
	
	auto it = iGamesContainer.iGameMap.begin(); 
	if (it != iGamesContainer.iGameMap.end())
	{
		if (iGamesContainer.IsGameSupported(it->second))
			iGameId = it->first;
		else
			FindNextSupportedGame(it->first);
	}
}

void GamesIter_const::End()
{
	iGameId.clear();
}

const TGame& GamesIter_const::operator*()
{
	auto it = iGamesContainer.iGameMap.find(iGameId);
	return it->second;
} 

GamesIter_const& GamesIter_const::operator++() 
{
	auto it = iGamesContainer.iGameMap.find(iGameId);
	iGameId.clear();	// after using it to find the name;
	if (it != iGamesContainer.iGameMap.end())
	{
		++it;
		if (it != iGamesContainer.iGameMap.end())
		{
			if (iGamesContainer.IsGameSupported(it->second))
				iGameId = it->first;
			else
				FindNextSupportedGame(it->first);
		}
	}
	return *this; 
}


bool GamesIter_const::equals(const GamesIter_const& aOther) const 
{
	return aOther.iGameId == iGameId; 
}

