/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "gameinfodialog.h"
#include "sdlthemeproperties.h"
#include "themeproperties.h"

static bool KillCharInChapterName(char c)
{
	switch(c)
	{
		case '-':
		case ' ':
			return true;
		default:
			return false;
	}
}

static void GetHeading(const std::string& aLine, std::string& aTextHeading)
{
	std::string line = aLine;
	line.erase(std::remove_if(line.begin(), line.end(), &KillCharInChapterName), line.end());
	aTextHeading.assign(line);
}


static bool SkipLineBasedOnFirstChar(char aBeginChar)
{
	return (aBeginChar == '$') || (aBeginChar == '\n') || (aBeginChar == '\r');
}


GameInfoDialog::GameInfoDialog(const std::string& aGameInfo) 
{
	ApplyProperties();
	std::string formattedGameInfoText;
	FormatGameInfoText(aGameInfo, formattedGameInfoText);
	iSimpleTextWidget.SetText(formattedGameInfoText.c_str());
}
	

void GameInfoDialog::ApplyProperties()
{
	SetInternalMargin(sdl::TMargin(sdl::SDLThemeProperties.iDefaultFont.iSize));
	sdl::TSize size(AppThemeProperties.iGameInfoDialogProperties.iWidth, AppThemeProperties.iGameInfoDialogProperties.iHeight);
	if (!size.IsEmpty())
		SetSize(size);
}
	
void GameInfoDialog::Setup()
{
	AddWidget(iSimpleTextWidget);
	Layout();
}
	
void GameInfoDialog::FormatGameInfoText(const std::string& aGameInfoText, std::string& aFormattedGameInfoText)
{
	std::istringstream stream(aGameInfoText);
	std::string line;
	
	bool firstLine = true;
	while( std::getline(stream, line))
	{
		if (line.length() > 0)
		{
			if (!SkipLineBasedOnFirstChar(line[0]))
			{
				if (firstLine)
				{
					aFormattedGameInfoText += AppThemeProperties.iGameInfoDialogProperties.iFormatTitle + line;
					aFormattedGameInfoText += "<p><br>" + AppThemeProperties.iGameInfoDialogProperties.iFormatBody;
					firstLine = false;
				}
				else
				{
					ProcessLine(line, aFormattedGameInfoText);
				}
			}
		}
	}
	
}
	
void GameInfoDialog::ProcessLine(const std::string& aLine, std::string& aFormattedGameInfoText)
{
	if (aLine[0] == '-')
	{
		std::string heading;
		GetHeading(aLine, heading);
		
		aFormattedGameInfoText += "<p><br><p><br>" + AppThemeProperties.iGameInfoDialogProperties.iFormatHeading + heading;
		aFormattedGameInfoText += "<p><br>" + AppThemeProperties.iGameInfoDialogProperties.iFormatBody;
	}
	else
	{
		aFormattedGameInfoText += aLine;
	}
}
