/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "gamemenus.h"
#include "sdlmenu2dmodel.h"
#include "gameenginemanager.h"

#include "userproperties.h"
#include "appmenudefs.h"


sdl::Menu* GameMenus::FindOrCreateMenu(TMenuLists& aMenuLists, const std::string& aCategory)
{
	auto menuIt = aMenuLists.find(aCategory);
	if (menuIt == aMenuLists.end())
		aMenuLists[aCategory] = new sdl::Menu(aCategory.c_str(), KMenuTypeGame.c_str());
	return aMenuLists[aCategory];
}


bool GameMenus::ShouldIncludeCategory(const std::string& aCategory)
{
	return std::find(AppUserProperties.iGameCategories.iExclude.begin(), AppUserProperties.iGameCategories.iExclude.end(), aCategory) == AppUserProperties.iGameCategories.iExclude.end();
}

bool GameMenus::ShouldIncludeGameEngine(const std::string& aGamenEngineName)
{
	return std::find(AppUserProperties.iGameEngines.iExclude.begin(), AppUserProperties.iGameEngines.iExclude.end(), aGamenEngineName) == AppUserProperties.iGameEngines.iExclude.end();
}

void GameMenus::Populate(TMenuLists& aMenuLists)
{
	bool updateCategoryNames = (iCategoryNames.size() == 0) ? true : false;
	iGameEngineNames.clear();
	
	for (auto &gameContainer : iGameEngineManager.GetGamesContainer())
	{
		const std::string& gameEngineName = gameContainer.first; 
		iGameEngineNames.push_back(gameEngineName);

		if (ShouldIncludeGameEngine(gameEngineName))
		{
			int count = 0;
			for (auto &game : gameContainer.second)
			{
				count++;
				FindOrCreateMenu(aMenuLists, game.iCategory)->AddMenuItem(new sdl::MenuItem(game.iDescription.c_str(), game.EncodeGameID().c_str()));
				
				if (updateCategoryNames)
					if (std::find(iCategoryNames.begin(), iCategoryNames.end(), game.iCategory) == iCategoryNames.end())
						iCategoryNames.push_back(game.iCategory);
			}
			std::string name = gameContainer.second.Name();
			std::transform(name.begin(), name.end(), name.begin(), ::toupper);
			printf("Based on game and category filtering, added %i games for game engine %s\n", count, name.c_str());

			if (updateCategoryNames)
				std::sort(iCategoryNames.begin(), iCategoryNames.end());
		}
	}
	
	std::sort(iGameEngineNames.begin(), iGameEngineNames.end());
}

void GameMenus::Construct()
{
	RePopulate();
}

void GameMenus::PopulateNoGamesMenu(TMenuLists& aMenuLists)
{
	std::string category("No games");
	sdl::Menu* emptyMenu = new sdl::Menu(category.c_str(), KMenuTypeGame.c_str());
	sdl::MenuItem* emptyMenuItem = new sdl::MenuItem("No game roms", "");
	emptyMenu->AddMenuItem(emptyMenuItem);
	aMenuLists[category] = emptyMenu;
}

void GameMenus::UpdateCategories()
{
	for (auto &menuName : iCategoryNames)
	{
		bool enable = ShouldIncludeCategory(menuName);
		iMenuModel.EnableMenu(menuName, enable);
	}
}

void GameMenus::RePopulate()
{
	iMenuModel.DeleteMenusByType(KMenuTypeGame);

	TMenuLists	menuLists;
	Populate(menuLists);
	
	if (menuLists.size() == 0)
		PopulateNoGamesMenu(menuLists);
	
	for (auto &menuList : menuLists )
	{
		menuList.second->Sort();
		iMenuModel.AddMenu(menuList.second);
	}
	
	UpdateCategories();
}
