/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __RESOURCE_FILE_SYSTEM_PROPERTIES_H
#define __RESOURCE_FILE_SYSTEM_PROPERTIES_H

#include "resourcefile.h"
#include "systemproperties.h"
#include "sdlsystemproperties.h"

class ResourceFileSystemProperties : public ResourceFile
{
public:
	static ResourceFileSystemProperties Instance;

	void AddMediaDirectory(const std::string& aName, TMediaDirectory& aMediaDirectory)
	{
		iResources.Add(aName + ".Path", aMediaDirectory.iPathName);
		iResources.Add(aName + ".Name", aMediaDirectory.iName);
		iResources.Add(aName+ ".IgnoreLeading", aMediaDirectory.iIgnoreLeading);
		iResources.Add(aName + ".IgnoreTrailing", aMediaDirectory.iIgnoreTrailing);
	}

	void AddGameEngine(const std::string& aName, TGameEngineProperties& aGameEngineProperties)
	{
		iResources.Add(aName + ".ExeName", aGameEngineProperties.iExeName);
		iResources.Add(aName + ".CommandLine", aGameEngineProperties.iCommandLine);
		iResources.Add(aName + ".IncludeAllRoms", aGameEngineProperties.iIncludeAllRoms);
		iResources.Add(aName + ".RomFileExtension", aGameEngineProperties.iRomFileExtension);
		iResources.Add(aName + ".MediaDirectory", aGameEngineProperties.iMediaDirectory);
		AddMediaDirectory(aName + ".RomDirectory", aGameEngineProperties.iRomDirectory);
		
		for (int i=0 ; i< KNumOfUserMediaDirectories ; i++)
			AddMediaDirectory(aName + ".MediaDirectory" + std::to_string(i+1), aGameEngineProperties.iMediaDirectories[i]);
	}
	
	void AddResources(sdl::SystemProperties& aSDLSystemProperties, SystemProperties& aSystemProperties)
	{
		Reset();

		iResources.Add("Renderer", aSDLSystemProperties.iRenderer);
		
		iResources.Add("Theme.Dir", aSystemProperties.iThemeDir);
		iResources.Add("Data.Dir", aSystemProperties.iDataDir);
		iResources.Add("User.Dir", aSystemProperties.iUserDir);
		iResources.Add("Font.Dirs", aSystemProperties.iFontDirs);
		iResources.Add("Image.Dirs", aSystemProperties.iImageDirs);
		
		for (auto &gameEngine : aSystemProperties.iGameEnginesProperties)
		{
			std::string name = gameEngine.first;
			std::transform(name.begin(), name.end(),name.begin(), ::toupper);
			AddGameEngine(name, gameEngine.second);
		}
	}
	
};

#endif // __RESOURCE_FILE_SYSTEM_PROPERTIES_H