/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __RESOURCE_FILE_THEME_PROPERTIES_H
#define __RESOURCE_FILE_THEME_PROPERTIES_H

#include "resourcefile.h"
#include "themeproperties.h"


inline void DefaultTitleHeight(int& aTitleHeight)
{
	sdl::SetDefaultPositiveValue(aTitleHeight, sdl::SDLThemeProperties.iDefaultWindowProperties.iLabelTitle.iFont.iSize * 2);
}

inline void DefaultFontName(std::string& aFontName)
{
	sdl::SetDefaultString(aFontName, sdl::SDLThemeProperties.iDefaultFont.iName);
}

inline void DefaultFontSize(int& aFontSize)
{
	sdl::SetDefaultPositiveValue(aFontSize, aFontSize = sdl::SDLThemeProperties.iDefaultFont.iSize);
}

inline void DefaultLargeFontSize(int& aFontSize)
{
	sdl::SetDefaultPositiveValue(aFontSize, sdl::SDLThemeProperties.iDefaultFont.iSize * 1.5);
}

inline void DefaultFontColor(sdl::TColor& aColor)
{
	aColor.SetDefault(sdl::SDLThemeProperties.iDefaultFont.iTextColor.iColor);
}

inline void DefaultFillColor(sdl::TColor& aColor)
{
	aColor.SetDefault(sdl::SDLThemeProperties.iDefaultWindowProperties.iRoundedRect.iFillColor);
	aColor.SetDefault(sdl::SDLThemeProperties.iDefaultBackgroundColor);
}

inline void DefaultRoundedRectRadiusX(int& aRadiusX)
{
	sdl::SetDefaultPositiveValue(aRadiusX, sdl::SDLThemeProperties.iDefaultWindowProperties.iRoundedRect.iRadiusX);
}

inline void DefaultRoundedRectRadiusY(int& aRadiusY)
{
	sdl::SetDefaultPositiveValue(aRadiusY, sdl::SDLThemeProperties.iDefaultWindowProperties.iRoundedRect.iRadiusY);
}

inline void DefaultFrameColor(sdl::TColor& aColor)
{
	aColor.SetDefault(sdl::SDLThemeProperties.iDefaultWindowProperties.iRoundedRect.iFrameColor);
}

inline void DefaultRoundedRectFrameWidth(int& aFrameWidth)
{
	sdl::SetDefaultPositiveValue(aFrameWidth, sdl::SDLThemeProperties.iDefaultWindowProperties.iRoundedRect.iFrameWidth);
}

class ResourceFileThemeProperties : public ResourceFile 
{
public:
	static ResourceFileThemeProperties Instance;
	
	void AddResources(sdl::ThemeProperties& aSDLThemeProperties, ThemeProperties& aThemeProperties)
	{
		Reset();
		
		AddResources(aSDLThemeProperties);
		AddResources(aThemeProperties);
	}
	
private:
	void AddPopup(const std::string& aName, sdl::PopupMenuWidget::Properties& aPopupMenuProperties)
	{
		AddLabel(aName + ".Text", aPopupMenuProperties.iTextLabels);
		AddRoundedRect(aName + ".RoundedRect", aPopupMenuProperties.iRoundedRect);
	}
	
	void AddWindow(const std::string& aName, sdl::Window::Properties& aWindowProperties)
	{
		AddLabel(aName + ".Title", aWindowProperties.iLabelTitle, true);
		AddRoundedRect(aName + ".RoundedRect", aWindowProperties.iRoundedRect);
		iResources.Add(aName + ".TitleHeight", aWindowProperties.iTitleHeight, DefaultTitleHeight, 1); // set default in second round
	}
	
	void AddFont(const std::string& aName, sdl::TFont& aFont, bool aLargeFont = false)
	{
		iResources.Add(aName + ".Name", aFont.iName, DefaultFontName);
		iResources.Add(aName + ".Size", aFont.iSize, aLargeFont ? DefaultLargeFontSize : DefaultFontSize);
		iResources.Add(aName + ".Outline", aFont.iOutline);
		iResources.Add(aName + ".Color", aFont.iTextColor, DefaultFontColor);
	}
	
	void AddMargin(const std::string& aName, sdl::TMargin& aMargin)
	{
		iResources.Add(aName + ".Left", aMargin.iLeft);
		iResources.Add(aName + ".Top", aMargin.iTop);
		iResources.Add(aName + ".Right", aMargin.iRight);
		iResources.Add(aName + ".Bottom", aMargin.iBottom);
	}
	
	void AddRoundedRect(const std::string& aName, sdl::TRoundedRect& aRoundedRect)
	{
		iResources.Add(aName + ".RadiusX", aRoundedRect.iRadiusX, DefaultRoundedRectRadiusX);
		iResources.Add(aName + ".RadiusY", aRoundedRect.iRadiusY, DefaultRoundedRectRadiusY);
		iResources.Add(aName + ".FrameColor", aRoundedRect.iFrameColor, DefaultFrameColor);
		iResources.Add(aName + ".FillColor", aRoundedRect.iFillColor, DefaultFillColor);
		iResources.Add(aName + ".FrameWidth", aRoundedRect.iFrameWidth, DefaultRoundedRectFrameWidth);
	}

	void AddLabel(const std::string& aName, sdl::LabelWidget::Properties& aLabelProperties, bool aLargeFont = false)
	{
		AddFont(aName + ".Font", aLabelProperties.iFont, aLargeFont);
		iResources.Add(aName + ".Font.Bold", aLabelProperties.iFont.iBold);
		iResources.Add(aName + ".Font.Italic", aLabelProperties.iFont.iItalic);
		iResources.Add(aName + ".Font.Underline", aLabelProperties.iFont.iUnderline);
		iResources.Add(aName + ".EclipseText", aLabelProperties.iEclipseText);
		iResources.Add(aName + ".HorizontalAlignment", aLabelProperties.iHorizontalAlignment);
		iResources.Add(aName + ".VerticalAlignment", aLabelProperties.iVerticalAlignment);
	}
	
	void AddSimpleText(const std::string& aName, ThemeProperties::SimpleTextProperties& aSimpleText)
	{
		iResources.Add(aName + ".Rect", aSimpleText.iRect);
		iResources.Add(aName + ".Text", aSimpleText.iText);
		iResources.Add(aName + ".VerticalAlign", aSimpleText.iProperties.iVerticalAlign);
		iResources.Add(aName + ".SingleLine", aSimpleText.iProperties.iSingleLine);
		iResources.Add(aName + ".EclipseText", aSimpleText.iProperties.iEclipseText);
	}

	void AddGameImage(const std::string& aName, ThemeProperties::GameImageProperties& aGameImage)
	{
		iResources.Add(aName + ".Rect", aGameImage.iRect);
		iResources.Add(aName + ".Directory", aGameImage.iLogicalDirectory);
		iResources.Add(aName + ".FileName", aGameImage.iFileName);
		iResources.Add(aName + ".DefaultImage", aGameImage.iDefaultImage);
		iResources.Add(aName + ".DrawMode", aGameImage.iDrawMode);
	}

	void AddGameVideo(const std::string& aName, ThemeProperties::GameVideoProperties& aGameVideo)
	{
		iResources.Add(aName + ".Rect", aGameVideo.iRect);
		iResources.Add(aName + ".Directory", aGameVideo.iLogicalDirectory);
	}
	
	void AddResources(ThemeProperties& aThemeProperties)
	{
		iResources.Add("BackgroundImage", aThemeProperties.iBackgroundImage);

		iResources.Add("GameInfoDialog.Width", aThemeProperties.iGameInfoDialogProperties.iWidth);
		iResources.Add("GameInfoDialog.Height", aThemeProperties.iGameInfoDialogProperties.iHeight);
		iResources.Add("GameInfoDialog.FormatTitle", aThemeProperties.iGameInfoDialogProperties.iFormatTitle);
		iResources.Add("GameInfoDialog.FormatHeading", aThemeProperties.iGameInfoDialogProperties.iFormatHeading);
		iResources.Add("GameInfoDialog.FormatBody", aThemeProperties.iGameInfoDialogProperties.iFormatBody);
		for (int i = 0; i< KNumOfSimpleTexts; i++)
			AddSimpleText("Text" + std::to_string(i + 1), aThemeProperties.iGameSimpleText[i]);

		AddGameVideo("VideoSnap", aThemeProperties.iGameVideoProperties);
		AddGameImage("Snap", aThemeProperties.iGameImageProperties[0]);
		for (int i = 1; i< KNumOfGameImages; i++)
			AddGameImage("GameImage" + std::to_string(i), aThemeProperties.iGameImageProperties[i]);

		AddSimpleText("CategoryText", aThemeProperties.iCategoryTextProperties);
		iResources.Add("CategoryText.SystemMenuText", aThemeProperties.iSystemMenuCategoryText);
		iResources.Add("About.ExtraText", aThemeProperties.iExtraAboutText);
	}

	void AddResources(sdl::ThemeProperties& aSDLThemeProperties) 
	{ 
		iResources.Add("Screen.Width", aSDLThemeProperties.iScreen.iWidth);
		iResources.Add("Screen.Height", aSDLThemeProperties.iScreen.iHeight);
		iResources.Add("Screen.FullScreen", aSDLThemeProperties.iScreen.iFullScreen);

		AddWindow("DefaultWindow", aSDLThemeProperties.iDefaultWindowProperties);
		AddPopup("DefaultPopup", aSDLThemeProperties.iDefaultPopupMenuProperties);
		
		iResources.Add("DefaultBackgroundColor", aSDLThemeProperties.iDefaultBackgroundColor);
		
		AddFont("DefaultFont", aSDLThemeProperties.iDefaultFont);
		iResources.Add("HighlightBackgroundColor", aSDLThemeProperties.iDefaultHighlightBackgroundColor);
		iResources.Add("HighlightTextColor", aSDLThemeProperties.iDefaultHighlightTextColor);
		
		AddMargin("DefaultDialog.Margin", aSDLThemeProperties.iDefaultDialogProperties.iMargin);
		iResources.Add("DefaultDialog.Width", aSDLThemeProperties.iDefaultDialogProperties.iWidth);
		iResources.Add("DefaultDialog.Height", aSDLThemeProperties.iDefaultDialogProperties.iHeight);
		iResources.Add("DefaultDialog.BackgroundMask", aSDLThemeProperties.iDefaultDialogProperties.iBackgroundMask);

		iResources.Add("Debug.ShowOutline", aSDLThemeProperties.iDebug.iShowOutline);
		iResources.Add("Debug.OutlineColor", aSDLThemeProperties.iDebug.iOutlineColor);
		
		iResources.Add("Controls.FrameColor", aSDLThemeProperties.iControlProperties.iFrameColor, DefaultFrameColor);
		iResources.Add("Controls.FrameSize", aSDLThemeProperties.iControlProperties.iFrameSize);
		iResources.Add("Controls.FillColorOn", aSDLThemeProperties.iControlProperties.iFillColorOn, DefaultFrameColor);
		iResources.Add("Controls.FillColorOff", aSDLThemeProperties.iControlProperties.iFillColorOff, DefaultFillColor);
		iResources.Add("Controls.ScrollbarThumb", aSDLThemeProperties.iControlProperties.iScrollbarThumb, DefaultFrameColor);
		
		AddRoundedRect("InfoPrint.RoundedRect", aSDLThemeProperties.iInfoPrint.iRoundedRect);
		AddLabel("InfoPrint", aSDLThemeProperties.iInfoPrint.iMessage);
		iResources.Add("InfoPrint.Timeout", aSDLThemeProperties.iInfoPrint.iTimeout);

		iResources.Add("2DMenu.Rect", aSDLThemeProperties.iDefaultMenu2DUIProperties.iRect);
		iResources.Add("2DMenu.HorizontalAnimation", aSDLThemeProperties.iDefaultMenu2DUIProperties.iHorizontalAnimation);
		iResources.Add("2DMenu.VerticalAnimation", aSDLThemeProperties.iDefaultMenu2DUIProperties.iVerticalAnimation);
		iResources.Add("2DMenu.HorizontalAnimationDelay", aSDLThemeProperties.iDefaultMenu2DUIProperties.iHorizontalAnimationDelay);
		iResources.Add("2DMenu.VerticalAnimationDelay", aSDLThemeProperties.iDefaultMenu2DUIProperties.iVerticalAnimationDelay);
		iResources.Add("2DMenu.EnableImages", aSDLThemeProperties.iDefaultMenu2DUIProperties.iEnableImages);

		AddFont("2DMenu.TextMenu.Font", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iFont);
		iResources.Add("2DMenu.TextMenu.Rect", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iArea);
		iResources.Add("2DMenu.TextMenu.Round", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iRound);
		iResources.Add("2DMenu.TextMenu.Items", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iItems);
		iResources.Add("2DMenu.TextMenu.TextStyle", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iTextStyle);
		iResources.Add("2DMenu.TextMenu.FontSizeSelected", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iFontSizeSelected);
		iResources.Add("2DMenu.TextMenu.FontSizeGradient", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iFontSizeGradient);
		iResources.Add("2DMenu.TextMenu.FontColorGradient", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iFontColorGradient);
		iResources.Add("2DMenu.TextMenu.UseHighlight", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iUseHighlight);
		AddRoundedRect("2DMenu.TextMenu.HighlightRect", aSDLThemeProperties.iDefaultMenu2DUIProperties.iTextListMenuProperties.iHighlightRoundedRect);
		
		iResources.Add("2DMenu.ImageMenu.Items", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iItems);
		iResources.Add("2DMenu.ImageMenu.SelectedItemX", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iSelectedItemX);
		iResources.Add("2DMenu.ImageMenu.PreviousItemTopLeftOffset", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iPreviousItemTopLeftOffset);
		iResources.Add("2DMenu.ImageMenu.NextItemTopLeftOffset", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iNextItemTopLeftOffset);
		iResources.Add("2DMenu.ImageMenu.ImageSize", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iImageSize);
		iResources.Add("2DMenu.ImageMenu.StackedImageRatio", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iStackedImageRatio);
		iResources.Add("2DMenu.ImageMenu.StackedWidth", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iStackWidth);
		iResources.Add("2DMenu.ImageMenu.Directory", aSDLThemeProperties.iDefaultMenu2DUIProperties.iImageListMenuProperties.iLogicalImageDirectory);
	}
	
	virtual void OnLoad() override { }
};

#endif // __RESOURCE_FILE_THEME_PROPERTIES_H
