/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "settingsdialogs.h"
#include "sdlthemeproperties.h"
#include "themeproperties.h"
#include "sdlinfoprint.h"
#include "sdllayout.h"
#include "sdllabelwidget.h"
#include "version.h"

void TestDialog::Setup()
{
	SettingsDialog::Setup();
#if 1	

#if 0
	for (int i=0 ; i<30 ; i++)
	{
		static bool switchValue[30];
		sdl::Switch* switch = new sdl::Switch;
		switch->SetLabelText(std::string("Enable switch ") + std::to_string(i));
		switch->SetEnabled(switchValue[i]);
		AddInput(*switch, true); // take ownership
#endif
#if 0
	for (int i=0 ; i<30 ; i++)
	{
		static sdl::Spinner::SpinnerModelInt spinnerModelInt[30];
		spinnerModelInt[i].SetRange(0, i);
		spinnerModelInt[i].SetValue(0);
		sdl::Spinner* spinner = new sdl::Spinner(spinnerModelInt[i]);
		spinner->SetLabelText(std::string("Spinner ") + std::to_string(i));
		AddInput(*spinner, true); // take ownership
	}
#endif
#if 1
	for (int i=0 ; i<30 ; i++)
	{
		static sdl::SpinnerWidget::SpinnerModelList spinnerModel[30];
		for (int j=0 ; j<i ; j++)
			spinnerModel[i].AddString(std::string("String ") + std::to_string(j));
		iSpinners[i] = new sdl::SpinnerWidget(spinnerModel[i]);
		iSpinners[i]->SetLabelText(std::string("Spinner ") + std::to_string(i));
		iSpinners[i]->SetMaxWidthValueString("Spinner spinner");
		AddInput(*iSpinners[i], true); // take ownership
	}
	
#endif
#endif

	// button bar with ok button
	iOkButton.SetText("OK");
	iOkButton.SetCommand(sdl::TCommand::EOK);
	AddButton(iOkButton);
	
	iCancelButton.SetText("Cancel");
	iCancelButton.SetCommand(sdl::TCommand::ECancel);
	AddButton(iCancelButton);

	Layout();
}


class SDLJoystickButtonInputWidget : public sdl::Widget, public sdl::IJoystickButtonEventHandler
{
public:
	SDLJoystickButtonInputWidget(const sdl::JoystickProperties::TButton& aButton) : iButton(aButton) 
	{ 
		StateChanged();
		
		iLabelText.SetAlignment(sdl::LabelWidget::Widget::THorizontalAlignment::ELeft);
		iLabelText.SetAlignment(sdl::LabelWidget::Widget::TVerticalAlignment::ECenter);
		iLabelText.SetMargin(sdl::TMargin((int) (sdl::SDLThemeProperties.iDefaultFont.iSize) / 2, 0, 0, 0));
		
		iLabelValue.SetAlignment(sdl::LabelWidget::Widget::THorizontalAlignment::ERight);
		iLabelValue.SetAlignment(sdl::LabelWidget::Widget::TVerticalAlignment::ECenter);
		iLabelValue.SetMargin(sdl::TMargin(0, 0, (int) (sdl::SDLThemeProperties.iDefaultFont.iSize) / 2, 0));
		
		iLabelText.SetText(iButton.iName);
	}
	
	const sdl::JoystickProperties::TButton& GetButton() { return iButton; }
	
protected:
	void Render(sdl::IRenderer& aRenderer)
	{
		Widget::Render(aRenderer);
		
		const sdl::TRect screenRect = GetScreenRect();
		
		iLabelText.SetTextColor(HasFocus() ? sdl::SDLThemeProperties.iDefaultHighlightTextColor : sdl::SDLThemeProperties.iDefaultFont.iTextColor);
		iLabelValue.SetTextColor(HasFocus() ? sdl::SDLThemeProperties.iDefaultHighlightTextColor : sdl::SDLThemeProperties.iDefaultFont.iTextColor);
		
		iLabelText.SetPosition(screenRect.Position());
		iLabelText.Render(aRenderer);
		
		iLabelValue.SetPosition(screenRect.Position());
		iLabelValue.Render(aRenderer);
	}
	
	virtual int GetMinimumHeight() const override
	{
		return (int) (iLabelText.GetFontSize() * 1.5);
	}
	
	void StateChanged()
	{
		iLabelValue.SetText(GetValueText());
	}
	
	virtual void FocusChanged() override
	{
		if (HasFocus())
		{
			iHasChanged = false;
			sdl::EventManager::SetJoystickButtonEventHandler(this);
		}
		else
		{
			sdl::EventManager::ResetJoystickButtonEventHandler();
		}
		StateChanged();
	}
	
	std::string GetValueText() 
	{
		std::string valueText;
		
		if (HasFocus() && !iHasChanged)
			valueText = "Press joystick button";
		else
			if ((iButton.iJoystickId == -1) || (iButton.iJoystickButtonId == -1))
				valueText = "none";
			else
				valueText = "joystick: " + std::to_string(iButton.iJoystickId) + ", button: " + std::to_string(iButton.iJoystickButtonId);

		return valueText;
	}
	
	virtual void ProcessJoyButtonEvent(const SDL_JoyButtonEvent& aEvent) override
	{
		if (HasFocus() && (aEvent.state == SDL_PRESSED))
		{
			iButton.iJoystickId = aEvent.which;
			iButton.iJoystickButtonId = aEvent.button;
			iHasChanged = true;
			StateChanged();
			sdl::System::RequestReRender();
		}
	}
	
	virtual void SizeChanged(const sdl::TSize& aOldSize) override { iLabelText.SetSize(GetSize()); iLabelValue.SetSize(GetSize()); sdl::Widget::SizeChanged(aOldSize); }
	virtual bool HasInput() const override { return true; }
	
	sdl::JoystickProperties::TButton	iButton;
	sdl::LabelWidget 					iLabelText;
	sdl::LabelWidget 					iLabelValue;
	bool 								iHasChanged = false;
};


void DialogConfigureJoystickButtons::Setup()
{
	sdl::SettingsDialog::Setup();
	
	for (auto &button : iButtons)
	{
		SDLJoystickButtonInputWidget* joystickButtonInputWidget = new SDLJoystickButtonInputWidget(button);
		AddInput(*joystickButtonInputWidget, true); // move ownership to container
		
		iJoystickButtonInputWidgets.push_back(joystickButtonInputWidget);
	}
	
	// button bar with ok button
	iOkButton.SetText("OK");
	iOkButton.SetCommand(sdl::TCommand::EOK);
	AddButton(iOkButton);

	Layout();
}

void DialogConfigureJoystickButtons::OnDialogExit(bool aSuccess)
{
	if (aSuccess)
		for (auto i=0 ; i<(int)iButtons.size() ; i++)
			iButtons[i] = iJoystickButtonInputWidgets[i]->GetButton();
}


/*******************************************
 * 
 * DialogConfigureJoystickNavigationSpeed
 * 
 ******************************************/


void DialogConfigureJoystickNavigationSpeed::Setup()
{
	sdl::SettingsDialog::Setup();
	
	iSpinnerModelScrollSpeed.SetRange(1, 10000);
	iSpinnerModelScrollSpeed.SetValue(iScrollSpeed);
	iScrollSpeedSpinner.SetLabelText("Scroll speed (milliseconds)");
	AddInput(iScrollSpeedSpinner);

	iSpinnerModelFastScrollSpeed.SetRange(1, 10000);
	iSpinnerModelFastScrollSpeed.SetValue(iFastScrollSpeed);
	iFastScrollSpeedSpinner.SetLabelText("Fast scroll speed (milliseconds)");
	AddInput(iFastScrollSpeedSpinner);

	iSpinnerModelScrollDelay.SetRange(1, 10000);
	iSpinnerModelScrollDelay.SetValue(iFastScrollDelay);
	iSpinnerModelScrollDelay.SetIncrement(10);
	iFastScrollDelaySpinner.SetLabelText("Fast scroll delay (milliseconds)");
	AddInput(iFastScrollDelaySpinner);
	
	// button bar with ok button
	iOkButton.SetText("OK");
	iOkButton.SetCommand(sdl::TCommand::EOK);
	AddButton(iOkButton);
	
	Layout();
}

void DialogConfigureJoystickNavigationSpeed::OnDialogExit(bool aSuccess)
{
	if (aSuccess)
	{
		iScrollSpeed = iSpinnerModelScrollSpeed.GetValue();
		iFastScrollSpeed = iSpinnerModelFastScrollSpeed.GetValue();
		iFastScrollDelay = iSpinnerModelScrollDelay.GetValue();
	}
}


/*******************************************
 *
 * DialogConfigureJoystickNavigation
 * 
 ******************************************/

void DialogConfigureJoystickNavigation::SetPressText(int aStep)
{
	iSwitches[aStep]->SetLabelText("Press joystick " + iButtonTexts[aStep]);
}
	
void DialogConfigureJoystickNavigation::SetDoneText(int aStep)
{
	iSwitches[aStep]->SetLabelText("Joystick " + iButtonTexts[aStep]);
}
	
void DialogConfigureJoystickNavigation::DoProcessJoyEvent(const SDL_JoyHatEvent& aEvent)
{
	bool proceed = true;
	if (iStep == 0)
		iPressedJoystickId = aEvent.which;
	else
		if (iPressedJoystickId != aEvent.which)
		{
			proceed = false;
			sdl::InfoPrint::Message("Ignored, different joystick detected");
		}
	
	if (proceed)
	{
		iSwitches[iStep]->SetEnabled(true);
		iSwitches[iStep]->SetPrivateData(aEvent.value);
		SetDoneText(iStep);
		SwitchFocusNext();
		iStep++;
		if (iStep < (int) iButtonTexts.size())
			SetPressText(iStep);
		sdl::System::RequestReRender();
	}
}

void DialogConfigureJoystickNavigation::ProcessJoyHatEvent(const SDL_JoyHatEvent& aEvent)
{
	if ((aEvent.value > 0) && (iStep < (int) iButtonTexts.size()))
		DoProcessJoyEvent(aEvent);
}

bool DialogConfigureJoystickNavigation::HandleCommand(sdl::TCommand aCommand)
{
	bool consumed = false;
	
	switch(aCommand)
	{
		case  sdl::TCommand::EUp:
		case  sdl::TCommand::ELeft:
		case  sdl::TCommand::EDown:
		case  sdl::TCommand::ERight:
		{
			consumed = true;
			break;
		}
		case  sdl::TCommand::ESelect:
		{
			if (iStep < (int) iButtonTexts.size())
				consumed = true;
			break;
		}
		default:
			break;
	}
	
	if (consumed)
		sdl::System::RequestReRender();
	else
		consumed = sdl::Dialog::HandleCommand(aCommand);
	
	return consumed;
}

void DialogConfigureJoystickNavigation::Setup()
{
	sdl::SettingsDialog::Setup();
	
	for (auto i=0 ; i<(int)iButtonTexts.size() ; i++)
	{
		sdl::SwitchWidget* switchWidget = new sdl::SwitchWidget();
		switchWidget->SetLabelText("<none>");
		switchWidget->SetOnText("SET");
		switchWidget->SetOffText("");
		switchWidget->SetEnabled(false);
		AddInput(*switchWidget, true); // move ownership to container
		
		iSwitches.push_back(switchWidget);
	}
	SetPressText(0);
	
	// button bar with ok button
	iOkButton.SetText("OK");
	iOkButton.SetCommand(sdl::TCommand::EOK);
	AddButton(iOkButton);
	
	Layout();
	
	sdl::EventManager::SetJoystickHatEventHandler(this);
}

void DialogConfigureJoystickNavigation::OnDialogExit(bool aSuccess)
{
	sdl::EventManager::ResetJoystickHatEventHandler();
	
	if (aSuccess)
	{
		for (int i=0 ; i< (int)iSwitches.size() ; i++)
		{
			iSwitches[i]->GetPrivateData(*iButtonIds[i]);
			iJoystickId = iPressedJoystickId;
		}
	}
}


/*******************************************
 * 
 * DialogConfigureSystemMenu
 * 
 ******************************************/

void DialogConfigureSystemMenu::Setup()
{
	SettingsDialog::Setup();
	
	iSwitch.SetLabelText("Enable \"" + iName + "\" menu");
	iSwitch.SetEnabled(iMenuEnabled);
	AddInput(iSwitch);
	
	iSpinner.SetLabelText("Max entries");
	iSpinnerModel.SetRange(1, 999);
	AddInput(iSpinner);
	
	// button bar with ok button
	iOkButton.SetText("OK");
	iOkButton.SetCommand(sdl::TCommand::EOK);
	AddButton(iOkButton);
	
	Layout();
}
	

void DialogConfigureSystemMenu::OnDialogExit(bool aSuccess)
{
	if (aSuccess)
	{
		iMenuEnabled = iSwitch.IsEnabled();
		iMenuMaxEntries = iSpinnerModel.GetValue();
	}
}


/*******************************************
 * 
 * DialogListOfSwitches
 * 
 ******************************************/

void DialogListOfSwitches::Setup()
{
	sdl::SettingsDialog::Setup();
	
	// add switches
	for (auto &attibute : iAttributes)
	{
		sdl::SwitchWidget* switchWidget = new sdl::SwitchWidget();
		switchWidget->SetLabelText(attibute);
		bool enabled = std::find(iAttributesEnabledOrDisabled.begin(), iAttributesEnabledOrDisabled.end(), (attibute)) != iAttributesEnabledOrDisabled.end();
		if (!iIsEnabled)
			enabled = !enabled;
		switchWidget->SetEnabled(enabled);
		AddInput(*switchWidget, true); // move ownership to container
		
		iSwitches.push_back(switchWidget);
	}
	
	sdl::ButtonWidget* button = new sdl::ButtonWidget;
	button->SetText("OK");
	button->SetCommand(sdl::TCommand::EOK);
	AddButton(*button, true);
	
	button = new sdl::ButtonWidget;
	button->SetText("Select all");
	button->SetCommand((sdl::TCommand) ECommandSelectAll);
	AddButton(*button, true);
	
	button = new sdl::ButtonWidget;
	button->SetText("Select none");
	button->SetCommand((sdl::TCommand) ECommandSelectNone);
	AddButton(*button, true);
	
	Layout();
}

void DialogListOfSwitches::OnDialogExit(bool aSuccess)
{
	if (aSuccess)
	{
		iAttributesEnabledOrDisabled.clear();
		for (auto &switchWidget : iSwitches)
		{
			bool enabled = !switchWidget->IsEnabled();
			if (iIsEnabled)
				enabled = !enabled;
			if (enabled)
				iAttributesEnabledOrDisabled.push_back(switchWidget->GetText());
		}
	}
}

bool DialogListOfSwitches::HandleCommand(sdl::TCommand aCommand)
{
	bool consumed = false;
	
	switch ((unsigned) aCommand)
	{
		case ECommandSelectAll:
		{
			CommandSelectAll();
			consumed = true;
			break;
		}
		case ECommandSelectNone:
		{
			CommandSelectNone();
			consumed = true;
			break;
		}
		default:
		{
			consumed = Dialog::HandleCommand(aCommand);
			break;
		}
	}

	if (consumed)
		sdl::System::RequestReRender();

	return consumed;
}


void DialogListOfSwitches::CommandSelectAll()
{
	std::for_each(iSwitches.begin(), iSwitches.end(), [=](sdl::SwitchWidget* aSwitch){aSwitch->SetEnabled(true);});
}

void DialogListOfSwitches::CommandSelectNone()
{
	std::for_each(iSwitches.begin(), iSwitches.end(), [=](sdl::SwitchWidget* aSwitch){aSwitch->SetEnabled(false);});
}

/*******************************************
 * 
 * DialogAbout
 * 
 ******************************************/


#define stringify1(s) #s
#define stringify(s) stringify1(s)

DialogAbout::DialogAbout() 
{
	std::string text = 
	"<p align=center>Retro Game Launcher (RGL), version " stringify(APP_VERSION)
	"<p align=center>(c) 2016, Peter van Sebille"
	"<p align=center><br>";

	std::string textHeader = 
	"<p align=center>peter@yipton.net, <font color=0xff><u>www.yipton.net</u>";
	
	if (AppThemeProperties.iExtraAboutText.size() > 0)
		text += AppThemeProperties.iExtraAboutText + "<p align=center><br>";

	text += textHeader;
	
	iSimpleTextView.SetText(text);
}
	
void DialogAbout::Setup()
{
	SetInternalMargin(sdl::TMargin(0, (int) (sdl::SDLThemeProperties.iDefaultFont.iSize * 1.5), 0, (int) (sdl::SDLThemeProperties.iDefaultFont.iSize * 1.5)));
	
	AddWidget(iSimpleTextView);
	Layout();
	
	ResizeToClientArea(sdl::TSize(GetSize().iWidth, iSimpleTextView.GetTotalHeight()));
}


/*******************************************
 * 
 * DialogAudioVolume
 * 
 ******************************************/

void DialogAudioVolume::Setup()
{
	if (iMute)
		sdl::InfoPrint::Message("Audio is muted - no audible feedback");

	sdl::SettingsDialog::Setup();
	
	iMuteSwitch.SetLabelText("Mute audio");
	iMuteSwitch.SetEnabled(iMute);
	AddInput(iMuteSwitch);

	iVolumeSlider.SetLabelText("Audio volume (%)");
	iVolumeSlider.SetValue(iVolume);
	AddInput(iVolumeSlider);
	
	// button bar with ok button
	iOkButton.SetText("OK");
	iOkButton.SetCommand(sdl::TCommand::EOK);
	AddButton(iOkButton);
	
	Layout();
}

void DialogAudioVolume::OnDialogExit(bool aSuccess)
{
	if (aSuccess)
	{
		iVolume = iVolumeSlider.GetValue();
		iMute = iMuteSwitch.IsEnabled();
	}
}

bool DialogAudioVolume::HandleCommandMyself(sdl::TCommand aCommand)
{
	bool consumed = true;
	// if the user enables audio mute then we should not let sdl::Dialog send a user feedback event - it will be handled(and thus audio feedback may be given)
	// before we enable the audio mute
	// Note that if the user enables audio then sdl:Dialog will also send the user feedback event before we change the audio settings (no feedback audio 
	// will be played pressing "ok"). I can live with that.
	if ((aCommand == sdl::TCommand::EOK) && iMuteSwitch.IsEnabled())
		Done(true);
	else
		consumed = Dialog::HandleCommandMyself(aCommand);
	
	return consumed;
}

/*******************************************
 * 
 * Video settings
 * 
 ******************************************/

void DialogVideoSettings::Setup()
{
	sdl::SettingsDialog::Setup();
	
	iVideoEnabledSwitch.SetLabelText("Enable video playback");
	iVideoEnabledSwitch.SetEnabled(iVideoEnabled);
	AddInput(iVideoEnabledSwitch);

	iMuteAudioSwitch.SetLabelText("Mute audio during video");
	iMuteAudioSwitch.SetEnabled(iMuteAudio);
	AddInput(iMuteAudioSwitch);
	
	
	iAudioVolumeDuringVideoSlider.SetLabelText("Audio volume (%)");
	iAudioVolumeDuringVideoSlider.SetValue(iAudioVolumeDuringVideo);
	AddInput(iAudioVolumeDuringVideoSlider);

	iStartDelaySpinner.SetLabelText("Playback delay (ms)");
	iStartDelaySpinnerModel.SetRange(100, 10000);
	iStartDelaySpinnerModel.SetIncrement(10);
	iStartDelaySpinnerModel.SetValue(iStartAfterDelay);
	AddInput(iStartDelaySpinner);

	iTimeLabelWitch.SetLabelText("Show time during playback");
	iTimeLabelWitch.SetEnabled(iTimeLabel);
	AddInput(iTimeLabelWitch);
	
	// button bar with ok button
	iOkButton.SetText("OK");
	iOkButton.SetCommand(sdl::TCommand::EOK);
	AddButton(iOkButton);
	
	Layout();
	
}

void DialogVideoSettings::OnDialogExit(bool aSuccess)
{
	if (aSuccess)
	{
		iVideoEnabled = iVideoEnabledSwitch.IsEnabled();
		iMuteAudio = iMuteAudioSwitch.IsEnabled();
		iAudioVolumeDuringVideo = iAudioVolumeDuringVideoSlider.GetValue();
		iStartAfterDelay = iStartDelaySpinnerModel.GetValue();
		iTimeLabel = iTimeLabelWitch.IsEnabled();
	}
}

