/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SETTINGS_DIALOGS_H
#define __SETTINGS_DIALOGS_H

#include "sdldialog.h"
#include "sdlimagewidget.h"
#include "sdlspinnerwidget.h"
#include "sdlswitchwidget.h"
#include "sdlbuttonwidget.h"
#include "sdlsliderwidget.h"
#include "types.h"
#include "sdlsimpletextwidget.h"
#include "sdljoyproperties.h"

class TestDialog : public sdl::SettingsDialog
{
public:
	TestDialog() {}
private:
	virtual void Setup() override;
	
	sdl::ButtonWidget 	iOkButton;
	sdl::ButtonWidget 	iCancelButton;
	sdl::SpinnerWidget*	iSpinners[30];
};

class GridContainer : public sdl::Container
{
protected:
	virtual int GetMinimumHeight() const override { return GetSize().iHeight; }
};

class SDLJoystickButtonInputWidget;

class DialogConfigureJoystickButtons : public sdl::SettingsDialog
{
public:
	DialogConfigureJoystickButtons(std::vector<sdl::JoystickProperties::TButton>& aButtons)	: iButtons(aButtons) { }
	
protected:
	virtual void Setup() override;
	virtual void OnDialogExit(bool aSuccess) override;
	
	sdl::ButtonWidget 								iOkButton;
	std::vector<sdl::JoystickProperties::TButton>&	iButtons;
	std::vector<SDLJoystickButtonInputWidget*>		iJoystickButtonInputWidgets;
};


class DialogConfigureJoystickNavigationSpeed : public sdl::SettingsDialog
{
public:
	DialogConfigureJoystickNavigationSpeed(int& aScrollSpeed, int& aFastScrollSpeed, int& aFastScrollDelay)
		: iScrollSpeedSpinner(iSpinnerModelScrollSpeed), iFastScrollSpeedSpinner(iSpinnerModelFastScrollSpeed), iFastScrollDelaySpinner(iSpinnerModelScrollDelay),
		iScrollSpeed(aScrollSpeed), iFastScrollSpeed(aFastScrollSpeed), iFastScrollDelay(aFastScrollDelay) { }

protected:
	virtual void Setup() override;
	virtual void OnDialogExit(bool aSuccess) override;

	sdl::ButtonWidget 	iOkButton;
	
	sdl::SpinnerWidget::SpinnerModelInt	iSpinnerModelScrollSpeed;
	sdl::SpinnerWidget::SpinnerModelInt	iSpinnerModelFastScrollSpeed;
	sdl::SpinnerWidget::SpinnerModelInt	iSpinnerModelScrollDelay;
	
	sdl::SpinnerWidget	iScrollSpeedSpinner;
	sdl::SpinnerWidget	iFastScrollSpeedSpinner;
	sdl::SpinnerWidget	iFastScrollDelaySpinner;
	
	int& 			iScrollSpeed;
	int& 			iFastScrollSpeed;
	int& 			iFastScrollDelay;
};


class DialogConfigureJoystickNavigation : public sdl::SettingsDialog, public sdl::IJoystickHatEventHandler
{
public:
	DialogConfigureJoystickNavigation(const std::vector<std::string>& aButtonTexts, std::vector<int*>& aButtonIds, int& aJoystickId) 
	: iButtonTexts(aButtonTexts), iButtonIds(aButtonIds), iJoystickId(aJoystickId) {}
protected:
	void SetPressText(int aStep);
	void SetDoneText(int aStep);
	void DoProcessJoyEvent(const SDL_JoyHatEvent& aEvent);
	
	virtual void ProcessJoyHatEvent(const SDL_JoyHatEvent& aEvent) override;
	
	virtual bool HandleCommand(sdl::TCommand aCommand) override;
	virtual void Setup() override;
	virtual void OnDialogExit(bool aSuccess) override;
	
	sdl::ButtonWidget 				iOkButton;
	std::vector<sdl::SwitchWidget*> iSwitches;
	int 							iStep = 0;
	int 							iPressedJoystickId = -1;
	const std::vector<std::string>& iButtonTexts;
	std::vector<int*> 				iButtonIds;
	int& 							iJoystickId;
};


class DialogListOfSwitches : public sdl::SettingsDialog
{
public:
	DialogListOfSwitches(
		const std::vector<std::string>& aAttributes, 
		std::vector<std::string>& aAttributesEnabledorDisabled,
		bool aIsEnabled) 
	: iAttributes(aAttributes), iAttributesEnabledOrDisabled(aAttributesEnabledorDisabled), iIsEnabled(aIsEnabled) {}
private:
	enum
	{
		ECommandSelectAll = unsigned(sdl::TCommand::EUserDefined),
		ECommandSelectNone = unsigned(sdl::TCommand::EUserDefined) + 1
	};
	virtual void Setup() override;
	void OnDialogExit(bool aSuccess) override;
	virtual bool HandleCommand(sdl::TCommand aCommand) override;
	
	void CommandSelectAll();
	void CommandSelectNone();
	
	const std::vector<std::string>& iAttributes;
	std::vector<std::string>&		iAttributesEnabledOrDisabled;
	bool 							iIsEnabled;
	std::vector<sdl::SwitchWidget*> iSwitches;
};

class DialogFilterCategories : public DialogListOfSwitches
{
public:
	DialogFilterCategories(const std::vector<std::string>& aCategories, std::vector<std::string>& aExcludeCategories) 
	:  DialogListOfSwitches(aCategories, aExcludeCategories, false) {}
};

class DialogFilterGameEngines : public DialogListOfSwitches
{
public:
	DialogFilterGameEngines(const std::vector<std::string>& aGameEngines, std::vector<std::string>& aExcludeGameEngines) 
	:  DialogListOfSwitches(aGameEngines, aExcludeGameEngines, false) {}
};



class DialogConfigureSystemMenu : public sdl::SettingsDialog
{
public:
	DialogConfigureSystemMenu(bool& aMenuEnabled, int& aMenuMaxEntries, const std::string& aName) 
	: iMenuEnabled(aMenuEnabled), iMenuMaxEntries(aMenuMaxEntries), iName(aName), iSpinnerModel(iMenuMaxEntries), iSpinner(iSpinnerModel) {}
private:
	virtual void Setup() override;
	void OnDialogExit(bool aSuccess) override;
	
	bool& 		iMenuEnabled;
	int& 		iMenuMaxEntries;
	std::string iName;
	
	sdl::ButtonWidget 					iOkButton;
	sdl::SwitchWidget					iSwitch;
	sdl::SpinnerWidget::SpinnerModelInt	iSpinnerModel;
	sdl::SpinnerWidget					iSpinner;
};

class DialogAbout : public sdl::Dialog
{
public:
	DialogAbout();
private:
	virtual void Setup() override;
	void OnDialogExit(bool aSuccess) override {}
	
	sdl::SimpleTextWidget iSimpleTextView;
};

class DialogAudioEvents : public DialogListOfSwitches
{
public:
	DialogAudioEvents(const std::vector<std::string>& aEvents, std::vector<std::string>& aEnabledEvents) 
	: DialogListOfSwitches(aEvents, aEnabledEvents, true) {}
};


class DialogAudioVolume : public sdl::SettingsDialog
{
public:
	DialogAudioVolume(int& aVolume, bool& aMute)  : iVolume(aVolume), iMute(aMute)  { }
private:
	virtual void Setup() override;
	virtual void OnDialogExit(bool aSuccess) override;
	virtual bool HandleCommandMyself(sdl::TCommand aCommand) override;
	
	int& 	iVolume;
	bool&	iMute;
	
	sdl::ButtonWidget 	iOkButton;
	sdl::SliderWidget 	iVolumeSlider;
	sdl::SwitchWidget 	iMuteSwitch;
};


class DialogVideoSettings : public sdl::SettingsDialog
{
public:
	DialogVideoSettings(bool& aVideoEnabled, bool& aMuteAudio, int& aAudioVolumeDuringVideo, int& aStartAfterDelay, bool& aTimeLabel)  
	: iVideoEnabled(aVideoEnabled), iMuteAudio(aMuteAudio), iAudioVolumeDuringVideo(aAudioVolumeDuringVideo), iStartAfterDelay(aStartAfterDelay),
	iTimeLabel(aTimeLabel), iStartDelaySpinner(iStartDelaySpinnerModel) { }
private:
	virtual void Setup() override;
	virtual void OnDialogExit(bool aSuccess) override;
	
	bool&	iVideoEnabled;
	bool&	iMuteAudio;
	int&	iAudioVolumeDuringVideo;
	int&	iStartAfterDelay;
	bool&	iTimeLabel;
	
	sdl::ButtonWidget 	iOkButton;
	sdl::SliderWidget 	iAudioVolumeDuringVideoSlider;
	sdl::SwitchWidget 	iMuteAudioSwitch;
	sdl::SwitchWidget 	iVideoEnabledSwitch;
	sdl::SwitchWidget 	iTimeLabelWitch;
	
	sdl::SpinnerWidget::SpinnerModelInt	iStartDelaySpinnerModel;
	sdl::SpinnerWidget					iStartDelaySpinner;
};


#endif // __SETTINGS_DIALOGS_H
