/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "themesettingsdecorator.h"
#include "themeproperties.h"
#include "appmenudefs.h"
#include "resourcefilethemeproperties.h"
#include "fileutils.h"
#include "thememodel.h"
#include "sdlmenu.h"

void ThemeSettingsDecorator::Construct()
{
	ConstructThemePreviewImage();
	ConstructLabel();
}

void ThemeSettingsDecorator::ConstructLabel()
{
	if (!AppThemeProperties.iGameImageProperties[0].iRect.IsEmpty())
	{
		sdl::TRect rect = AppThemeProperties.iGameImageProperties[0].iRect;
		rect.SetHeight(iThemePreviewLabel.GetMinimumHeight());
		iThemePreviewLabel.SetRect(rect);
		iThemePreviewLabel.SetAlignment(THorizontalAlignment::ECenter);
		AddWidget(iThemePreviewLabel);
	}
}

void ThemeSettingsDecorator::ConstructThemePreviewImage()
{
	if (!AppThemeProperties.iGameImageProperties[0].iRect.IsEmpty())
	{
		iThemePreviewImage.SetName("ThemePreviewmage");
		iThemePreviewImage.OverrideDefaults(AppThemeProperties.iGameImageProperties[0]);
		iThemePreviewImage.SetShowDebugOutline();
		AddWidget(iThemePreviewImage);
	}
}


void ThemeSettingsDecorator::MenuItemHighlighted(const sdl::MenuItem& aMenuItem)
{
	std::string fileName;
	sdl::TSize size;
	
	GetThemeInfo(aMenuItem.GetCommand(), fileName, size);
	std::string labelText = size.IsEmpty() ? "Unknown size" : std::to_string((int)size.iWidth) + "x" + std::to_string((int)size.iHeight);
	UpdateThemePreview(fileName, labelText);
}

	

bool ThemeSettingsDecorator::GetThemeInfo(const std::string& aThemeName, std::string& aBackgroundImageFilename, sdl::TSize& aScreenSize)
{
	sdl::ThemeProperties	sdlThemeProperties;
	ThemeProperties			themeProperties;
	ResourceFileThemeProperties	resourceFileProperties;
	
	aBackgroundImageFilename.clear();
	aScreenSize= sdl::TSize(0, 0);
	
	resourceFileProperties.AddResources(sdlThemeProperties, themeProperties);
	bool success = resourceFileProperties.Load(iThemeModel.GetThemeFileName(), iThemeModel.GeThemeDirName(aThemeName));
	if (success)
	{
		std::vector<std::string> imageDirs = sdl::System::ImageDirs.GetDirectoryNames();
		imageDirs.insert(imageDirs.begin(), iThemeModel.GeThemeDirName(aThemeName));
		FileUtils::FindFileInDirs(themeProperties.iBackgroundImage, imageDirs, aBackgroundImageFilename);
		aScreenSize = sdl::TSize(sdlThemeProperties.iScreen.iWidth, sdlThemeProperties.iScreen.iHeight);
	}
	return success;
}


void ThemeSettingsDecorator::UpdateThemePreview(const std::string& aFileName, const std::string& aLabelText)
{
	iThemePreviewImage.SetImageFile(aFileName);
	iThemePreviewLabel.SetText(aLabelText);
}

void ThemeSettingsDecorator::MenuChanged(sdl::Menu& aMenu)
{
	UpdateThemePreview("", "");
}