/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __THEME_SETTINGS_DECORATOR_H
#define __THEME_SETTINGS_DECORATOR_H

#include "sdlimagewidget.h"
#include "sdlcontainer.h"
#include "sdllabelwidget.h"
#include "sdlimenulistener.h"
#include "types.h"

class ThemeModel;

class ThemeSettingsDecorator : public sdl::Container, public sdl::IMenuItemListener, public sdl::IMenuChangedListener
{
public:
	ThemeSettingsDecorator(ThemeModel& aThemeModel) : iThemeModel(aThemeModel) {}
	virtual ~ThemeSettingsDecorator() { }
	void Construct();
	
private: 
	// implements IMenuItemListener
	virtual void MenuItemHighlighted(const sdl::MenuItem& aMenuItem) override;
	virtual void MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand) override { }
	virtual void MenuListChanged(sdl::Menu& aMenu) override {}
	
	// implements
	virtual void MenuChanged(sdl::Menu& aMenu) override;
	
	void ConstructLabel();
	void ConstructThemePreviewImage();
	void UpdateThemePreview(const std::string& aFileName, const std::string& aLabelText);
	bool GetThemeInfo(const std::string& aThemeName, std::string& aBackgroundImageFilename, sdl::TSize& aScreenSize);
	
	sdl::ImageWidget iThemePreviewImage;
	sdl::LabelWidget iThemePreviewLabel;
	
	ThemeModel& iThemeModel;
};

#endif // __THEME_SETTINGS_DECORATOR_H