/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __FILE_UTILS_H
#define __FILE_UTILS_H

#include "types.h"
#include <dirent.h>
#include <functional>

class IFileStore;

template<class TContainer>
void Store(TContainer& aContainer, std::string& aKey, std::string& aValue);

class FileUtils
{
public:
	static int ReadFile(const std::string& aFileName, TStringList& aStringList, char aSeperator = '=');
	static int ReadFile(const std::string& aFileName, TStringMap& aStringMap, char aSeperator = '=');
	static int ReadFile(const std::string& aFileName, TStringPairList& aStringPairList, char aSeperator = '=');
	
	static int ReadDirectory(const std::string& aDirName, TStringList& aStringList);
	static int ReadDirectory(const std::string& aDirName, TStringList& aStringList, const std::string& aFileExtension);
	static int ReadDirectory(const std::string& aDirName, TStringList& aStringList, const std::function <bool (struct dirent*, std::string& aOutName)>& aWantEntryFunction);
	
	static bool WriteFile(const std::string& aFileName, const TStringPairList& aStringPairList);
	static bool WriteFile(const std::string& aFileName, const TStringMap& aStringMap);
	
	static bool DirectoryExists(const std::string& aDirName);
	static bool FileExists(const std::string& aFileName);
	
	static void FindFile(const std::string& aFileName, const std::string& aDirectoryName, std::string& aFoundFileName);
	static bool FindFileInDirs(const std::string& aFileName, const std::vector<std::string> aDirs, std::string& aFoundFileName);
	
	static void ReadFilePart(const std::string& aFileName, int aOffset, int aLength, std::string& aContent);
	
private:
	static int ReadFile(const std::string& aFileName, IFileStore& aFileStore, char aSeperator = '=');
public:
	template<class TContainer>
	static int ReadFileNew(const std::string& aFileName, TContainer& aContainer)
	{
		std::ifstream file(aFileName.c_str(), std::ios::in);
		if (!file.good())
		{
			std::cout << "ERROR: could not open " << aFileName << ". Bailing out.\n";
			return 0;
		}
		
		std::string line;
		int count = 0;
		while (std::getline(file, line))
		{
			count++;
			
			if (line.size() == 0)
				continue;
			
			std::istringstream stream(line);
			std::string key;
			std::string value;
			
			if(std::getline(stream, key, '='))
				std::getline(stream, value);
			else
				key.assign(line);

			//		printf("File(%d): %s, key=%s, value=%s\n", count, aFileName.c_str(), key.c_str(), value.c_str());
			::Store<TContainer>(aContainer, key, value);
		}
		return count;
	}
	
};

#endif // __FILE_UTILS_H