/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __STRING_UTILS_H
#define __STRING_UTILS_H

#include "types.h"

class StringUtils
{
public:
	enum class TSubstituteOptions
	{
		EFirst,
		EAll
	};
	
	typedef std::map<std::string, std::string> TSubstituteMap;
	static void Substitute(std::string& aText, const TSubstituteMap& aSubstituteMap);
	static bool Substitute(std::string& aText, const std::string& aMatchString, const std::string& aSubstituteString, TSubstituteOptions aSubstituteOptions = TSubstituteOptions::EFirst);
	
	static bool StripQuotes(std::string& aString);
	static void SplitString(const std::string& aString, std::vector<std::string>& aStringList);
	static bool GetBoolValueFromString(const std::string& aValue, bool& aBool);
	static bool IsSame(const std::string& a, const std::string& b);
	static void StripWhiteSpaces(std::string& aString);
	static std::string GetPureAlphaNumName(const std::string& aString);
	static void ToLower(std::string& aString);
	static void ParseCommandLine(const std::string& aString, std::vector<std::string>& aStringList);
	
	
	class ITextMetrics
	{
	public:
		virtual void GetTextSize(const std::string& aText, int& aWidth, int& aHeight) = 0;
	};
	
	static void FitText(ITextMetrics& aTextMetrics, const std::string& aText, std::string& aReturnText, int aMaxWidth, int& aWidth, int& aHeight, bool aEclipse);
};

#endif // __STRING_UTILS_H
