/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "platform.h"

#include <unistd.h>
#include <sys/wait.h>

#include <dirent.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

std::string Platform::PathSeperator()
{
	return "/";
}

bool Platform::RunProgram(const std::string& aExeName, const std::vector<std::string>&  aArgList)
{
	pid_t pid = fork();
	bool success = true;
	
	if (pid == 0)
	{
		// child
		for (auto &s : aArgList)
			printf("ARG: %s\n", s.c_str());
		
		const char* argList[aArgList.size() + 2];
		argList[0] = aExeName.c_str();
		for (int i=0 ; i<(int)aArgList.size() ; i++)
			argList[i+1] = aArgList.at(i).c_str();
		argList[aArgList.size() + 1] = nullptr;
		
		int ret = execv(aExeName.c_str(), (char* const*) argList);
		
		exit(ret);
	}
	else 
	{	
		// parent
		if (pid == -1)
			success = false;
		else
		{
			int status = 0;
			waitpid(-1, &status, 0);
			success = (status==0);
		}
	}
	return success;
}

bool Platform::RunProgram(const std::string&  aExeName, const std::string&  aParameter)
{
	printf("RUN: %s %s\n", aExeName.c_str(), aParameter.c_str());
	
	pid_t pid = fork();
	bool success = true;
	
	if (pid == 0)
	{
		// child
		int ret = execl(aExeName.c_str(), aExeName.c_str(), aParameter.c_str(), nullptr);
		exit(ret);
	}
	else 
	{	
		// parent
		if (pid == -1)
			success = false;
		else
		{
			int status = 0;
			waitpid(-1, &status, 0);
			success = (status==0);
		}
	}
	return success;
}


std::string Platform::GetPlatformName()
{
	return "linux"; // should be unix but linux is more convenient
}


bool Platform::FileExists(const std::string& aFileName)
{
	struct stat info;

	bool exists = false;
	if (stat(aFileName.c_str(), &info) == 0)
		if (S_ISREG(info.st_mode))
			exists = true;

	return exists;
}


bool Platform::DirectoryExists(const std::string& aDirName)
{
	struct stat info;

	bool exists = false;
	int ret = stat(aDirName.c_str(), &info);
	if (ret == 0)
		if (S_ISDIR(info.st_mode))
			exists = true;

	return exists;
}

void Platform::Shutdown()
{
	std::string command("/sbin/shutdown");
	execl(command.c_str(), command.c_str(), "-h", "now", nullptr);
}
