/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "platform.h"
#include <string>

#include <windows.h>
#include <stdio.h>
#include <tchar.h>

static std::string Basename(const std::string& aFilename)
{
	std::string basename = aFilename;
	size_t start = basename.find_last_of('\\');
	if (start != std::string::npos)
		basename.erase(basename.begin() + start, basename.end());

	return basename;
}

std::string Platform::PathSeperator()
{
	return "\\";
}

bool Platform::RunProgram(const std::string&  aExeName, const std::string&  aParameter)
{
	bool success = true;
	STARTUPINFOA		si;
	PROCESS_INFORMATION pi;

	ZeroMemory(&si, sizeof(si));
	si.cb = sizeof(si);
	ZeroMemory(&pi, sizeof(pi));

	std::string commandLine("\"" + aExeName + "\" " + aParameter);
	std::string dir(Basename(aExeName));

	if (CreateProcessA(
		nullptr,						// No module name (use command line)
		(LPSTR) commandLine.c_str(),
		nullptr,						// Process handle not inheritable
		nullptr,						// Thread handle not inheritable
		0,								// No handle inheritance
		0,								// Creation flags
		nullptr,						// Environment block
		dir.c_str(),					// Current working directory 
		&si,
		&pi)
		)
	{
		WaitForSingleObject(pi.hProcess, INFINITE);

		CloseHandle(pi.hProcess);
		CloseHandle(pi.hThread);
	}
	else
	{
		printf("ERROR: could not launch MAME (%s), CreateProcess failed (%d)\n", commandLine.c_str(), GetLastError());
		success = false;
	}
	return success;
}

bool Platform::RunProgram(const std::string& aExeName, const std::vector<std::string>&  aArgList)
{
	std::string param;

	for (auto &arg : aArgList)
		param += arg + " ";
	
	return RunProgram(aExeName, param);
}


std::string Platform::GetPlatformName()
{
	return "win32";
}

static bool IsFilenameAbsolute(const std::string& aFileName)
{
	bool isFilenameAbsolute = false;
	if (aFileName.size() >= 2)
		if (aFileName.at(1) == ':')
			isFilenameAbsolute = true;
	else if (aFileName.size() >= 1)
	{
		if (aFileName.at(0) == '\\')
			isFilenameAbsolute = true;
	}

	return isFilenameAbsolute;
}

static std::string GetAbsoluteFilename(const std::string& aFileName)
{
	std::string filename;

	if (IsFilenameAbsolute(aFileName))
		filename.assign(aFileName);
	else
	{
		char path[MAX_PATH];
		if (GetCurrentDirectoryA(MAX_PATH, (LPSTR)&path[0]) == 0)
			printf("Warning: GetCurrentDirectory failed, error=0x%x\n", GetLastError());

		filename.assign(std::string(path) + Platform::PathSeperator() + aFileName);
	}
	return filename;
}


bool Platform::FileExists(const std::string& aFileName)
{
	std::string fileName = GetAbsoluteFilename(aFileName);

	DWORD attrib = GetFileAttributesA((LPCSTR)fileName.c_str());
	return (attrib != INVALID_FILE_ATTRIBUTES && !(attrib & FILE_ATTRIBUTE_DIRECTORY));
}


bool Platform::DirectoryExists(const std::string& aDirName)
{
	std::string fileName = GetAbsoluteFilename(aDirName);

	DWORD attrib = GetFileAttributesA((LPCSTR)fileName.c_str());
	return (attrib != INVALID_FILE_ATTRIBUTES && (attrib & FILE_ATTRIBUTE_DIRECTORY));
}

void Platform::Shutdown()
{
	// nothing
}