/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlbase.h"
#include "commonsdlrenderer.h"

//#include "fileutils.h"
//#include "stringutils.h"
#include "nanosvg.h"
#include "nanosvgrast.h"
//#include "sdlthemeproperties.h"
//#include "sdluserproperties.h"

using namespace sdl;

SDL_Surface* sdl::CreateSurfaceFromSVG(const std::string& aSVGText, void** aSurfaceData)
{
	SDL_Surface* surface = nullptr;
	
	char* text = strdup(aSVGText.c_str());
	if (text == nullptr)
		return nullptr;
	
	struct NSVGimage* svgImage = nsvgParse(text, "px", 96);
	if (svgImage)
	{
		int width = svgImage->width;
		int height = svgImage->height;
		
		*aSurfaceData = malloc(width * height * 4);
		if (*aSurfaceData)
		{
			int pixelDataPitch = width * 4;
			
			struct NSVGrasterizer* svgRasterizer = nsvgCreateRasterizer();
			if (svgRasterizer)
			{
				nsvgRasterize(svgRasterizer, svgImage, 0, 0, 1, (unsigned char*) *aSurfaceData, width, height, pixelDataPitch);
				
				surface = SDL_CreateRGBSurfaceFrom(*aSurfaceData, width, height, 32, pixelDataPitch, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000);

				nsvgDeleteRasterizer(svgRasterizer);
			}
			else
				printf("CreateSurfaceFromSVG nsvgCreateRasterizer failed\n");
			
		}
		nsvgDelete(svgImage);
	}
	else
		printf("CreateSurfaceFromSVG nsvgParse failed\n");
	
	free(text);
	
	return surface;
}

SDL_Surface* sdl::CreateSurfaceFromTTF(TTF_Font& aFont, const std::string& aText, const TColor& aColor)
{
	if (aText.size() == 0)
		return nullptr;
	
	return TTF_RenderText_Blended(&aFont, aText.c_str(), aColor);
}

SDL_Surface* sdl::CreateSurfaceFromFile(const std::string& aFileName)
{
	SDL_Surface* surface = IMG_Load(aFileName.c_str());
	return surface;
}