/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlapplication.h"
#include "sdluserfeedbackmanager.h"
#include "sdlsystemrenderer.h"

using namespace sdl;

Application::Application(const std::string& aName) : iName(aName)
{
	System::Init();
}


Application::~Application()
{
	iJoystick.Close();
	
	System::Shutdown();
}

void Application::Construct()
{
	iApplicationWidget.Construct();
	
	iAudio.Construct();
	sdl::UserFeedbackManager::RegisterUserFeedbackHandler(iAudio);
	
	UserFeedbackManager::SendUserFeedbackEvent(UserFeedbackManager::EUserStartup);
	
	iJoystick.Construct();
	
	iEventManager.reset(CreateEventManager());
}

void Application::Run()
{
	SystemRenderer::Instance.Create(GetName());
	
	iEventManager->ClearEvents();
	iEventManager->EventLoop(iApplicationWidget);
	
	iApplicationWidget.ResetSDLState();
	SystemRenderer::Instance.Destroy();
}

