/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlbuttonwidget.h"
#include "sdlthemeproperties.h"

using namespace sdl;

ButtonWidget::ButtonWidget(TCommand aCommand) : iCommand(aCommand)
{
	int width = SDLThemeProperties.iDefaultFont.iSize * 8;
	SetSize(width, GetMinimumHeight());
	iLabel.SetAlignment(LabelWidget::THorizontalAlignment::ECenter);
	iLabel.SetAlignment(LabelWidget::TVerticalAlignment::ECenter);
}


void ButtonWidget::Render(IRenderer& aRenderer)
{
	TRect screenRect = GetScreenRect();
	if (HasFocus())
	{
		aRenderer.RenderFillRect(screenRect, SDLThemeProperties.iDefaultHighlightBackgroundColor);
		iLabel.SetTextColor(SDLThemeProperties.iDefaultHighlightTextColor);
	}
	else
	{
		RenderUtils::RenderRect(aRenderer, screenRect, SDLThemeProperties.iControlProperties.iFrameColor, SDLThemeProperties.iControlProperties.iFrameSize);
		iLabel.SetTextColor(SDLThemeProperties.iDefaultFont.iTextColor);
	}
	
	iLabel.SetPosition(screenRect.Position());
	iLabel.Render(aRenderer);
}

int ButtonWidget::GetMinimumHeight() const
{
	return (int) (iLabel.GetFontSize() * 1.5);
}

bool ButtonWidget::HandleCommand(TCommand aCommand)
{
	bool consumed = false;
	
	if (HasFocus())
	{
		if (aCommand == TCommand::ESelect)
		{
			System::RequestReRender();
			EventManager::SendCommand(iCommand);
			consumed = true;
		}
	}
	
	return consumed;
}

