/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_CONTAINER_H
#define __SDL_CONTAINER_H

#include "sdlcore.h"
#include "sdlwidget.h"


namespace sdl
{
	class Container : public Widget
	{
	public:
		enum THighlightChange
		{
			EUp,
			EDown,
			ETop,
			EBottom
		};
		Container() { }
		~Container();
		
		void AddWidget(Widget& aWidget, bool aTakeOwnership = false) { iWidgets.push_back(&aWidget); aWidget.SetParent(this); aWidget.SetParentOwnsMe(aTakeOwnership); }
		void RemoveWidget(Widget& aWidget);
		void RemoveAllWidgets();
		
		virtual void Render(IRenderer& aRenderer) override;
		virtual void ResetSDLState() override;
		
		virtual void Layout() override;
		
		int GetFocussedItemIndex() const	{ return iFocussedWidgetIndex; }
		int GetNumberOfWidgets() const 		{ return iWidgets.size(); }
		
	protected:
		virtual bool HandleCommand(TCommand aCommand) override;
		bool DelegateCommandToChildren(TCommand aCommand);
		bool DelegateCommandToFocussedWidget(TCommand aCommand);
		virtual bool HandleCommandMyself(TCommand aCommand);
		
		void LayoutChidlren();
		void LayoutSingleChild(Widget& aWidget);

		virtual Widget* FindFocusNext() override;
		virtual Widget* FindFocusPrevious() override;
		bool SwitchFocusNext();
		bool SwitchFocusPrevious();
		bool SwitchFocus(Widget* aWidget);
		bool HasFocussedWidget() const { return iFocussedWidgetIndex>=0; }
		
		int 		iFocussedWidgetIndex = -1;
		Widget*		iCurrentFocussedWidget = nullptr;
		
		TSize  GetClientAreaSize() const { return GetClientArea().Size(); }
		virtual TPoint GetRelativeChildPosition() const override { return GetClientArea().Position(); }
		
		virtual TRect GetClientArea() const { return TRect(GetSize()); }
		
		void ResizeToClientArea(const TSize& aClientAreaSize);
		
		virtual void SizeChanged(const TSize& aOldSize) override;
		virtual void PositionChanged(const TPoint& aOldPosition) override;
		
		virtual void HightlightChanged(THighlightChange aHighlightChange) { }
		
		std::vector<Widget*>	iWidgets;
		
		friend class LayoutManager;
	};
}

#endif // __SDL_CONTAINER_H