/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_IMAGE_WIDGET_H
#define __SDL_IMAGE_WIDGET_H

#include "sdlcore.h"
#include "sdlwidget.h"

namespace sdl
{
	class ImageWidget : public Widget
	{
	public:
		enum class TDrawMode
		{
			ENormal = 0,	// image drawn at iPosition
			EScaled			// image drawn scaled to fit iRect (keeping aspect ratio)
		};
		
		class Properties
		{
		public:
			Properties(const TRect& aRect = TRect(), const std::string& aDrawMode = "normal") : iRect(aRect), iDrawMode(aDrawMode) {}
			TRect 		iRect;
			std::string iDrawMode;
		};
		
		ImageWidget() { SetDefaultProperties(); }
		
		void SetImageFile(const std::string& aFileName)	{ iFileName = aFileName; CacheClear(); }
		void SetMargin(const TMargin& aMargin)			{ iMargin = aMargin; CacheClear(); }
		void SetDrawMode(TDrawMode aDrawMode);
		void OverrideDefaults(const Properties& aOther);
		virtual void Render(IRenderer& aRenderer) override;
		
	protected:
		virtual void ResetSDLState();
		
		void SetDefaultProperties()						{ ApplyProperties(); }
		void ApplyProperties()							{ SetRect(iImageProperties.iRect); SetDrawMode(iImageProperties.iDrawMode); }
		void SetDrawMode(const std::string& aDrawMode);

		std::shared_ptr<ITexture> GetSharedImage(IRenderer& aRenderer);
		
		std::string iFileName;
		Properties	iImageProperties;
		TDrawMode	iDrawMode = TDrawMode::ENormal;
		TMargin		iMargin;
	};
}

#endif // __SDL_IMAGE_WIDGET_H