/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_JOY_H
#define __SDL_JOY_H

#include <string>
#include "sdljoyproperties.h"
#include "sdleventmanager.h"

namespace sdl
{
	class JoystickHatEventHandler : public IJoystickHatEventHandler
	{
	public:
		JoystickHatEventHandler() {}
		virtual ~JoystickHatEventHandler();
		
	public: // implements IJoystickHatEventHandler
		virtual void ProcessJoyHatEvent(const SDL_JoyHatEvent& aEvent) override;
			
	private:
		friend Uint32 JoyCallback(Uint32 aInterval, void* aParam);
		void RepeatEvents();
		void GenerateCommand(TCommand aCommand);
		void StartGeneratingEvents(bool aFastScrolling = false);
		void StopGeneratingEvents();
		
		bool IsHatUp(int aHatCode)		{ return (aHatCode & SDLJoystickProperties.Navigation.iHatUp); }
		bool IsHatDown(int aHatCode)	{ return (aHatCode & SDLJoystickProperties.Navigation.iHatDown); }
		bool IsHatLeft(int aHatCode)	{ return (aHatCode & SDLJoystickProperties.Navigation.iHatLeft); }
		bool IsHatRight(int aHatCode)	{ return (aHatCode & SDLJoystickProperties.Navigation.iHatRight); }
		
		int iEventHatCode = 0;
		
		class TScroller
		{
		public:
			void Start() { iScrollTimeStart = SDL_GetTicks(); isFastScrolling = false; }
			bool ShouldFastScrollingBeStarted()
			{
				bool shouldFastScrollingBeStarted = false;
				if (!isFastScrolling && ((int)SDL_GetTicks() - iScrollTimeStart > SDLJoystickProperties.Navigation.Speed.iFastScrollDelayInMs))
					shouldFastScrollingBeStarted = isFastScrolling = true;
				return shouldFastScrollingBeStarted;
			}
		private:
			int iScrollTimeStart = 0;
			bool isFastScrolling = false;
		};
		
		TScroller		iScroller;
	};
	
	class JoystickButtonEventHandler : public IJoystickButtonEventHandler
	{
	public:
		public: // implements IJoystickButtonEventHandler
			virtual void ProcessJoyButtonEvent(const SDL_JoyButtonEvent& aEvent) override;
	};
	
	class Joystick
	{
	public:
		Joystick();
		virtual ~Joystick()	{ }
		void Construct()	{ Open(); }
		void Close();
	private:
		void Open();
		std::vector<SDL_Joystick*> 	iSdlJoysticks;
		
		
		JoystickHatEventHandler 	iJoystickHatEventHandler;
		JoystickButtonEventHandler	iJoystickButtonEventHandler;
	};
}

#endif // __SDL_JOY_H