/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdllistwidget.h"
#include "sdlthemeproperties.h"

using namespace sdl;

void ListWidget::ListItemWidget::Render(IRenderer& aRenderer)
{
	Widget::Render(aRenderer);
	
	TRect screenRect = GetScreenRect();
	
	iLabel.SetTextColor(HasFocus() ? SDLThemeProperties.iDefaultHighlightTextColor : SDLThemeProperties.iDefaultFont.iTextColor);
	iLabel.SetPosition(screenRect.Position());
	iLabel.Render(aRenderer);
}


void ListWidget::PreLayout()
{
	RemoveAllWidgets();
	
	for (auto &item : iListItems)
	{
		ListItemWidget* widget = new ListItemWidget();
		widget->GetLabel().SetText(item);
		widget->GetLabel().SetAlignment(LabelWidget::TVerticalAlignment::ECenter);
		
		AddWidget(*widget, true); // transfer ownership
	}
}

TSize ListWidget::CalculateMaxSize()
{
	TSize maxSize;
	for (auto &widget : iWidgets)
	{
		ListItemWidget* listItemWidget = (ListItemWidget*) widget;
		
		// width: how label calculates space needed to fit GetTextSize
		// height: height it got after Layout (i.e. its GetMinimumHeight())
		TSize size(listItemWidget->GetLabel().GetTextSize().iWidth, widget->GetHeight());
		maxSize.iHeight += size.iHeight;
		if (size.iWidth > maxSize.iWidth)
			maxSize.iWidth = size.iWidth;
	}
	return maxSize;
}