/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_MENU_2D_MODEL_H
#define __SDL_MENU_2D_MODEL_H

#include "types.h"
#include "sdlmenu.h"

namespace sdl 
{
	class Menu2DModel
	{
	public: 
		Menu2DModel() {}
		virtual ~Menu2DModel() {}

		Menu& CurrentMenu();

		static Menu* ConstructMenu(const std::string& aName, const std::string& aType, const TStringPairList& aStringPairList);
		static void GetMenuList(const Menu& aMenu, TStringPairList& aStringPairList);
		
		void AddMenuItemListener(IMenuItemListener& aMenuItemListener, const std::string& aMenuType = "");
		void AddMenuChangedListener(IMenuChangedListener& aMenuChangedListener);
		
		void HighlightMenuItem(const std::string& aMenuName, const std::string& aMenuItem);
		void GetHighlightMenuItem(std::string& aMenuName, std::string& aMenuItem);
		
		void AddMenu(Menu* aMenu);
		void DeleteMenusByType(const std::string& aType);
		void EnableMenu(const std::string& aName, bool aEnable);
		
		bool ActivatePrevMenu() { return DoSwitchMenu(-1); }
		bool ActivateNextMenu() { return DoSwitchMenu(1); }
		Menu* ActivateMenu(const std::string& aMenuName);
		
		bool SelectItem();
		bool GoBack();

	private:
		bool DoSwitchMenu(int aDelta);
		void ActivateMenu(Menu* aMenu);
		Menu* FindMenu(const std::string& aMenuName, int& aMenuIndex);
		
		void MenuChanged();
		
		std::vector<std::unique_ptr<Menu>>	iMenus;
		int									iCurrentMenu = -1;
		Menu*								iSubMenu = nullptr;
		
		std::vector<IMenuChangedListener*>	iMenuChangedListeners;
		std::map<std::string, std::set<IMenuItemListener*>>	iMenuItemListeners;
	};
}

#endif // __SDL_MENU_2D_MODEL_H