/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_RENDERER_INTERFACE_H
#define __SDL_RENDERER_INTERFACE_H

#include "sdlbase.h"

/*
 * Abstract interface to support both SDL and SDLGPU rendering
 * 
 */

namespace sdl
{
	class ITexture
	{
	public:
		virtual ~ITexture() {}
		virtual TRect GetRect() = 0;
		virtual void SetColor(const TColor& aColor) = 0;
		virtual void CopyBytes(void* aBytes, int aBytesPerRow) = 0;
	};
	
	
	class IRenderer
	{
	public:
		virtual ~IRenderer() {}
		virtual ITexture* 	CreateTextureFromSVG(const std::string& aSVGText) = 0;
		virtual ITexture* 	CreateTextureFromTTF(TTF_Font& aFont, const std::string& aText, const TColor& aColor) = 0;
		virtual ITexture*	CreateTextureFromFile(const std::string& aFileName) = 0;
		
		virtual ITexture*	CreateMemoryTexture(const TSize& aSize, const char* aChromaFormat = nullptr) = 0;
		virtual ITexture*	CreateRenderingTexture(const TSize& aSize) = 0;
		
		// must pass in a texture returned by CreateRenderingTexture()
		virtual IRenderer*	CreateRenderer(ITexture& aTexture) = 0;
		
		virtual void 	RenderTexture(ITexture& aTexture, const TPoint& aPoint) = 0;
		virtual void 	RenderTextureRotated(ITexture& aTexture, const TPoint& aPoint, float aDegrees=0) = 0;
		virtual void 	RenderTextureScaled(ITexture& aTexture, const TPoint& aPoint, float aScaleX, float aScaleY) = 0;

		virtual void 	RenderFillRect(const TRect& aRect, const TColor& aColor) = 0;
		virtual void 	RenderRect(const TRect& aRect, const TColor& aColor) = 0;
		
		virtual TRect	SetClipRect(const TRect& aRect) = 0;

		virtual void 	Clear() = 0;
	protected:
		friend class SystemRenderer;
		virtual void Present() = 0;
	};
	
} // namespace sdl

#endif // __SDL_RENDERER_INTERFACE_H