/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlsliderwidget.h"
#include "sdlthemeproperties.h"

using namespace sdl;

SliderWidget::SliderWidget()
{
	iLabel.SetAlignment(LabelWidget::THorizontalAlignment::ELeft);
	iLabel.SetAlignment(LabelWidget::TVerticalAlignment::ECenter);
	iLabel.SetMargin(TMargin((int) (SDLThemeProperties.iDefaultFont.iSize) / 2, 0, 0, 0));
}

bool SliderWidget::HandleCommand(TCommand aCommand)
{
	bool consumed = false;
	
	if (HasFocus())
	{
		switch(aCommand)
		{
			case TCommand::ELeft:
			{
				SetPercentage(iPercentage - iIncrease);
				consumed = true;
				break;
			}
			case TCommand::ERight:
			{
				SetPercentage(iPercentage + iIncrease);
				consumed = true;
				break;
			}
			default:
				break;
		}
	}
	if (consumed)
		System::RequestReRender();
	
	return consumed;
}

void SliderWidget::SetPercentage(int aPercentage)
{
	if (aPercentage>=0 && aPercentage<=100)
	{
		iPercentage = aPercentage;
		StateUpdated();
	}
}

void SliderWidget::CreateCachedTextures(IRenderer& aRenderer)
{
	TRect rect;
	CreateCachedFrameTexture(aRenderer, rect);
	CreateCachedTextureButton(aRenderer, rect);
}

void SliderWidget::CreateCachedFrameTexture(IRenderer& aRenderer, TRect& aRect)
{
	TRoundedRect roundedRect;
	roundedRect.iFillColor = SDLThemeProperties.iControlProperties.iFillColorOn;
	roundedRect.iFrameColor = SDLThemeProperties.iControlProperties.iFrameColor;
	roundedRect.iFrameWidth = 3;
	roundedRect.iRadiusX = 10;
	roundedRect.iRadiusY = 10;
	
	ITexture* texture = RenderUtils::CreateSVGDuoColorFilledRoundedRectTexture(aRenderer, GetSliderSpace(), roundedRect, SDLThemeProperties.iControlProperties.iFillColorOff, iPercentage);
	if (texture)
	{
		int rightMargin = GetMinimumHeight();
		const TRect widgetRect = GetSize();
		const TRect space(widgetRect.Position() + TPoint(widgetRect.Width()/2 - rightMargin, 0), TSize(widgetRect.Width()/2 + rightMargin, widgetRect.Height()));
		aRect = texture->GetRect();
		aRect.Center(space);
		
		CacheAdd(*texture, aRect.Position());
	}
}

void SliderWidget::CreateCachedTextureButton(IRenderer& aRenderer, const TRect& aRect)
{
	TColor frameColor = SDLThemeProperties.iControlProperties.iFrameColor;
	TColor fillColor = 	SDLThemeProperties.iControlProperties.iFrameColor; 
	
	int r = aRect.Height()/2;
	TSize size(TSize(r*2));
	
	ITexture* texture = RenderUtils::CreateSVGCircleTexture(aRenderer, size, 2, frameColor, fillColor);
	
	if (texture)
	{
		const TRect sliderRect(aRect);
		TRect rect = texture->GetRect();
		rect.SetPosition(sliderRect.Position() + TPoint((sliderRect.Width() * iPercentage)/100 - r, (sliderRect.Height() - r*2)/2));
		
		if (rect.X() < aRect.X())
			rect.SetPosition(TPoint(aRect.X(), rect.Y()));
		int delta = rect.GetTopRight().iX - aRect.GetTopRight().iX;
		if (delta > 0)
			rect -= TPoint(delta, 0);
		CacheAdd(*texture, rect.Position());
	}
}




void SliderWidget::Render(IRenderer& aRenderer)
{
	Widget::Render(aRenderer);
	
	const TRect screenRect = GetScreenRect();
	
	iLabel.SetTextColor(HasFocus() ? SDLThemeProperties.iDefaultHighlightTextColor : SDLThemeProperties.iDefaultFont.iTextColor);
	
	iLabel.SetPosition(screenRect.Position());
	iLabel.Render(aRenderer);
}


TSize SliderWidget::GetSliderSpace() const
{
	TSize size = GetSize(); 
	
	size.iWidth/=2; 
	size.iHeight *= .75; 
	
	return size; 
}

