/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_SLIDER_WIDGET_H
#define __SDL_SLIDER_WIDGET_H

#include "sdlcore.h"
#include "sdllabelwidget.h"


namespace sdl
{
	class SliderWidget : public Widget
	{
	public:
		SliderWidget();
		~SliderWidget() {}
		
		void SetLabelText(const std::string& aText)		{ iLabel.SetText(aText); }
		void SetIncrease(int aIncrease)					{ iIncrease = aIncrease; }
		void SetValue(int aPercentage)					{ iPercentage = aPercentage; StateUpdated(); }

		int  GetValue() const 							{ return iPercentage; }
		virtual bool HasInput() const override			{ return true; }
		virtual int GetMinimumHeight() const override	{ return (int) (iLabel.GetFontSize() * 1.5); }
		
	protected:
		virtual bool HandleCommand(TCommand aCommand) override;
		virtual void Render(IRenderer& aRenderer) override;
		virtual void SizeChanged(const TSize& aOldSize) override { iLabel.SetSize(GetSize()); Widget::SizeChanged(aOldSize); }
		void StateUpdated() { CacheClear(); }
		
		TSize GetSliderSpace() const;

		void SetPercentage(int aPercentage);
		
		LabelWidget	iLabel;
		int 		iPercentage = 0;
		int			iIncrease = 5;
		
		virtual void CreateCachedTextures(IRenderer& aRenderer) override;
		void CreateCachedFrameTexture(IRenderer& aRenderer, TRect& aRect);
		void CreateCachedTextureButton(IRenderer& aRenderer, const TRect& aRect);
	};
}



#endif // __SDL_SLIDER_WIDGET_H