/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "simpletextview.h"

using namespace simpletext;

static const char* AlignToText(TParaFormat::TAlignment aAlign)
{
	switch (aAlign)
	{
		case TParaFormat::TAlignment::ELeft:
			return "left";
			break;
		case TParaFormat::TAlignment::ECenter:
			return "center";
			break;
		case TParaFormat::TAlignment::ERight:
			return "right";
			break;
		case TParaFormat::TAlignment::EJustified:
			return "justified";
			break;
		default:
			break;
	}
	return "unknown";
}

void Dump(const TLineInfos& aLineInfos)
{
	int size = aLineInfos.size();
	printf("Number of lines: %d\n", size);
	for (int i=0 ; i<size ; ++i)
	{
		const TLineInfo& lineInfo = aLineInfos.at(i);
		printf("Line %d: width=%d, height=%d, descent=%d, align=%s\n", i, lineInfo.iWidth, lineInfo.iHeight, lineInfo.iLineDescent, AlignToText(lineInfo.iParaFormat.iAlign));
		
		int nrFragments = lineInfo.iLineFragments.size();
		for (int f=0 ; f< nrFragments ; ++f)
		{
			const TLineFragment& lineFragment = lineInfo.iLineFragments.at(f);
			printf("  fragment %d: words=%d, b=%s, u=%s, h=%d, f=%s, descent=%d\n", f, (int) lineFragment.iWords.size(), lineFragment.iCharFormat.iBold?"yes":"no", lineFragment.iCharFormat.iUnderline?"yes":"no",
				   lineFragment.iCharFormat.iFontSize, lineFragment.iCharFormat.iFontName.c_str(), lineFragment.iDescent);
			printf("    words:");
			for (int w=0 ; w<(int)lineFragment.iWords.size() ; w++)
			{
				printf(" \"%s\"", lineFragment.iWords[w].c_str());
			}
			printf("\n");
		}
	}
}

void View::Layout(int aWidth, IFontProvider& aFontProvider)
{
	iModel.Layout(aWidth, aFontProvider);
//	Dump(iModel.GetLineInfos());
}

int View::GetRenderHeight(int aStartLine, int aNumberOfLines)
{
	int height = 0;
	for (int i=aStartLine ; i< aStartLine+aNumberOfLines ; i++)
	{
		height += iModel.GetLineInfos()[i].iHeight;
	}
	return height;
}

void View::RenderLine(IViewRenderer& aViewRenderer, int aLine, int aX, int aY, int aWidth)
{
	const TLineInfos& lineInfos = iModel.GetLineInfos();
	const TLineInfo& lineInfo = lineInfos.at(aLine);
	
	int lineX = aX;
	if (lineInfo.iParaFormat.iAlign == TParaFormat::TAlignment::ERight)
		lineX += aWidth - lineInfo.iWidth;
	if (lineInfo.iParaFormat.iAlign == TParaFormat::TAlignment::ECenter)
		lineX += (aWidth - lineInfo.iWidth)/2;
	
	// for EAlignJustified
	int nrWordsOnLine = lineInfo.GetNrWords();
	float extraPixelGapPerWord = (nrWordsOnLine>1) ? ((float) aWidth - lineInfo.iWidth) / (float) (nrWordsOnLine-1) : (float) 0;
	int pixelsConsumed = 0;
	int wordCount = 0;
	
	for (int f=0 ; f<(int)lineInfo.iLineFragments.size() ; f++)
	{
		const TLineFragment& iLineFragment = lineInfo.iLineFragments.at(f);
		
		for (int w=0 ; w<(int)iLineFragment.iWords.size() ; w++)
		{
			const std::string& word = iLineFragment.iWords.at(w);
			int wordX = lineX;
			int wordY = aY + lineInfo.iHeight;
//			int wordY = aY + lineInfo.iHeight + (lineInfo.iLineDescent - iLineFragment.iDescent); //hmm seems descent business is getting in the way
			int wordWidth = aViewRenderer.RenderWord(wordX, wordY, word, iLineFragment.iCharFormat);
			
			wordCount++;
			lineX += wordWidth + iLineFragment.iSpaceWidth;
			if ((lineInfo.iParaFormat.iAlign == TParaFormat::TAlignment::EJustified) && !lineInfo.iIsLastLineInParagraph)
			{
				int shouldConsume = (int) (extraPixelGapPerWord * (float) wordCount);
				lineX += (shouldConsume - pixelsConsumed);
				pixelsConsumed = shouldConsume;
			}
		}
	}
	
}
