/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __VLC_WIDGET_H
#define __VLC_WIDGET_H

#include "sdlwidget.h"
#include "sdllabelwidget.h"

class VlcWidgetAudioControl
{
public:
	static void SetMute(bool aMute);
	static bool GetMute();
	static void SetAudioLevel(int aAudioLevel);
	static int 	GetAudioLevel();
};


class VlcWidget : public sdl::Widget 
{
public:
	VlcWidget();
	~VlcWidget();
	
	bool PlayVideoFile(const std::string& aFileName);
	void Stop();
	bool IsPlaying();
	void ShowTimeLabel(bool aEnabled) { iTimeLabel.SetEnabled(aEnabled); } 
	
private:
	virtual void Render(sdl::IRenderer& aRenderer) override;
	virtual void ResetSDLState() override { Widget::ResetSDLState(); Stop(); }
	
	void SetTimeLabel();
	void GetVideoTime(int& aDuration, int& aElapsedTime);
	
	void CopyFrameToTexture();
	void CreateTextureIfNeeded(sdl::IRenderer& aRenderer);
	
	void Lock(void** aPixels);
	void Unlock();

	static void* LockCallback(void* aContext, void** aPixels);
	static void UnlockCallback(void* aContext, void* /*aId*/, void* const* /*aPixels*/);
	
	void SetupAudioVolume();
	
	sdl::ITexture*	iTexture 		= nullptr;
	
	SDL_mutex*		iMutex 			= nullptr;
	void*			iVlcPlayer 		= nullptr;
	
	char* 			iVideoBuffer	= nullptr;
	int				iVideoBufferSize = 0;
	sdl::TSize 		iVideoSize;
	
	int 			iVideoDuration;
	int				iVideoStartTime;
	
	sdl::LabelWidget iTimeLabel;
};

#endif // __VLC_WIDGET_H